@@ -1,6 +1,6 @@
 ;;; treemacs-diagnostics.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -74,15 +74,15 @@ features several pre-made faces named `treemacs-diagnostic-*-face'.
 
 This method is debounced, it will never run more often than once every 3
 seconds.  In addition the use of a lazy thunk ensures that potentially expensive
-transformations happen only once and only when required.  Performance is thus not
-expected to be a major issue.
+transformations happen only once and only when required.  Performance is thus
+not expected to be a major issue.
 
 A basic example use would look like this:
 
 \(treemacs-apply-diagnostics
-    (thunk-delay '(\"/path/to/file/x\" 'treemacs-diagnostic-warning-face
-                   \"/path/to/file/y\" 'treemacs-diagnostic-error-face
-                   \"/path/to/file/z\" '((:underline \"green\")))))"
+    (thunk-delay \\='(\"/path/to/file/x\" \\='treemacs-diagnostic-warning-face
+                      \"/path/to/file/y\" \\='treemacs-diagnostic-error-face
+                      \"/path/to/file/z\" \\='((:underline \"green\")))))"
   (treemacs-debounce treemacs--diagnostic-timer treemacs--apply-diagnostics-delay
     (treemacs-run-in-every-buffer
      (save-excursion
