@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -46,8 +46,8 @@ Collapsed directories require special handling since all directories of a series
 need to be put under watch so as to be notified when the collapsed structure
 needs to change, but removing the file watch is not straightforward:
 
-Assume a series of directories are collapsed into one as '/c1/c2/c3/c4' and a
-new file is created in '/c1/c2'.  A refresh is started and only '/c1/c2' is
+Assume a series of directories are collapsed into one as \"/c1/c2/c3/c4\" and a
+new file is created in \"/c1/c2\".  A refresh is started and only \"/c1/c2\" is
 collapsed now, c3 and c4 are no longer part of the treemacs view and must be
 removed from the filewatch list.  However the event that triggered the refresh
 was one of a file being created, so it is not possible to know that c3 and c4
@@ -158,7 +158,7 @@ An event counts as relevant when
                        (let* ((file (caddr ,event))
                               (parent (treemacs--parent-dir file))
                               (cache (ht-get treemacs--git-cache parent)))
-                         (and cache (not (string= "!" (ht-get cache file))))))
+                         (and cache (eq 'treemacs-git-ignored-face (ht-get cache file)))))
                   (let* ((dir (caddr ,event))
                          (filename (treemacs--filename dir)))
                     (--any? (funcall it filename dir) treemacs-ignored-file-predicates)))))))))
