@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -38,12 +38,11 @@
    `(left-fringe ,treemacs--fringe-indicator-bitmap treemacs-fringe-indicator-face))
   "The `before-string' property value used by the fringe indicator overlay.")
 
-(define-inline treemacs--move-fringe-indicator-to-point ()
+(defun treemacs--move-fringe-indicator-to-point ()
   "Move the fringe indicator to the position of point."
-  (inline-quote
-   (when treemacs--fringe-indicator-overlay
-     (-let [pabol (point-at-bol)]
-       (move-overlay treemacs--fringe-indicator-overlay pabol  (1+ pabol))))))
+  (when treemacs--fringe-indicator-overlay
+    (-let [pabol (line-beginning-position)]
+      (move-overlay treemacs--fringe-indicator-overlay pabol (1+ pabol)))))
 
 (defun treemacs--enable-fringe-indicator ()
   "Enabled the fringe indicator in the current buffer."
@@ -76,8 +75,11 @@ WINDOW is the treemacs window that has just been focused or unfocused."
 
 (defun treemacs--tear-down-fringe-indicator-mode ()
   "Tear down `treemacs-fringe-indicator-mode'."
+  (remove-hook 'treemacs-mode-hook
+               #'treemacs--enable-fringe-indicator-in-current-buffer)
   (treemacs-run-in-all-derived-buffers
    (treemacs--disable-fringe-indicator)
+   (advice-remove #'hl-line-highlight #'treemacs--move-fringe-indicator-to-point)
    (remove-hook 'window-selection-change-functions
                 #'treemacs--show-fringe-indicator-only-when-focused
                 :local)))
@@ -118,12 +120,20 @@ fringe indicator when the treemacs window is selected."
                           (s-join "-")
                           (intern))))
   (setf treemacs-fringe-indicator-mode arg)
+  (add-hook 'treemacs-mode-hook
+            #'treemacs--enable-fringe-indicator-in-current-buffer)
   (treemacs-run-in-all-derived-buffers
-   (treemacs--enable-fringe-indicator)
-   (when (memq arg '(t only-when-focused))
-     (add-hook 'window-selection-change-functions
-               #'treemacs--show-fringe-indicator-only-when-focused
-               nil :local))))
+   (treemacs--enable-fringe-indicator-in-current-buffer)))
+
+(defun treemacs--enable-fringe-indicator-in-current-buffer ()
+  "Set up fringe-indicator-mode for the current buffer."
+  (treemacs--enable-fringe-indicator)
+  (advice-add #'hl-line-highlight
+              :after #'treemacs--move-fringe-indicator-to-point)
+  (when (memq treemacs-fringe-indicator-mode '(t only-when-focused))
+    (add-hook 'window-selection-change-functions
+              #'treemacs--show-fringe-indicator-only-when-focused
+              nil :local)))
 
 (treemacs-only-during-init (treemacs-fringe-indicator-mode))
 
