@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -29,48 +29,13 @@
 (require 'ht)
 (require 'treemacs-themes)
 (require 'treemacs-logging)
+(require 'treemacs-scope)
 
 (eval-when-compile
   (require 'cl-lib)
   (require 'inline)
   (require 'treemacs-macros))
 
-;; An explanation for the what and why of the icon highlighting code below:
-;; Using png images in treemacs has one annoying visual flaw: they overwrite the overlay
-;; used by hl-line, such that the line marked by hl-line will always show a 22x22 pixel
-;; gap wherever treemacs places an icon, regardess of transparency.
-;; Using xpm instead of png images is one way to work around this, but it degrades icon
-;; quality to an unacceptable degree. Another way is to directly change images' :background
-;; property. The backgrounds colours are derived from the current theme with `treemacs--setup-icon-highlight'
-;; and saved in `treemacs--selected-icon-background' and `treemacs--not-selected-icon-background'.
-;; Every icon string stores two images with the proper :background values in its properties
-;; 'img-selected and 'img-unselected. The 'display property of the icon in the current line
-;; is then highlighted, and the previously highlighted icon unhighlighted, by advising
-;; `hl-line-highlight'. The last displayed icon is saved as a button marker in `treemacs--last-highlight'.
-;; Since it is a marker in the treemacs buffer it is important for it to be reset whenever it might
-;; become invalid.
-
-(eval-and-compile
-  (defvar treemacs--not-selected-icon-background
-    (pcase (face-attribute 'default :background nil t)
-      ((or 'unspecified 'unspecified-bg "unspecified" "unspecified-bg")
-       (unless (or noninteractive (boundp 'treemacs-no-load-time-warnings))
-         (message "[Treemacs] Warning: coudn't find default background colour for icons, falling back on #2d2d31."))
-       "#2d2d31" )
-      (other other)))
-  "Background for non-selected icons.")
-
-(eval-and-compile
-  (defvar treemacs--selected-icon-background
-    (-let [bg (face-attribute 'hl-line :background nil t)]
-      (if (member bg '(unspecified unspecified-b "unspecified" "unspecified-bg"))
-          (prog1 treemacs--not-selected-icon-background
-            (unless (or noninteractive (boundp 'treemacs-no-load-time-warnings))
-              (message "[Treemacs] Warning: couldn't find hl-line-mode's background color for icons, falling back on %s."
-                       treemacs--not-selected-icon-background)))
-        bg)))
-  "Background for selected icons.")
-
 (define-inline treemacs--set-img-property (image property value)
   "Set IMAGE's PROPERTY to VALUE."
   ;; the emacs26 code where this is copied from says it's for internal
@@ -115,33 +80,6 @@ account."
          (plist-get it :background)
        (face-attribute ,face :background nil t)))))
 
-(defun treemacs--setup-icon-background-colors (&rest _)
-  "Align icon backgrounds with current Emacs theme.
-Fetch the current Emacs theme's background & hl-line colours and inject them
-into the gui icons of every theme in `treemacs--themes'.
-Also called as advice after `load-theme', hence the ignored argument."
-  (let* ((default-background (treemacs--get-local-face-background 'default))
-         (hl-line-background (treemacs--get-local-face-background 'hl-line))
-         (test-icon          (treemacs-get-icon-value 'dir-open))
-         (icon-background    (treemacs--get-img-property (get-text-property 0 'img-unselected test-icon) :background))
-         (icon-hl-background (treemacs--get-img-property (get-text-property 0 'img-selected test-icon) :background)))
-    (when (memq default-background '(unspecified-bg unspecified))
-      (treemacs-log-failure "Current theme fails to specify default background color, falling back on #2d2d31")
-      (setq default-background "#2d2d31"))
-    ;; make sure we only change all the icons' colors when we have to
-    (unless (and (string= default-background icon-background)
-                 (string= hl-line-background icon-hl-background))
-      (setf treemacs--selected-icon-background hl-line-background
-            treemacs--not-selected-icon-background default-background)
-      (dolist (theme treemacs--themes)
-        (treemacs--maphash (treemacs-theme->gui-icons theme) (_ icon)
-          (treemacs--set-img-property
-           (get-text-property 0 'img-selected icon)
-           :background treemacs--selected-icon-background)
-          (treemacs--set-img-property
-           (get-text-property 0 'img-unselected icon)
-           :background treemacs--not-selected-icon-background))))))
-
 (define-inline treemacs--is-image-creation-impossible? ()
   "Will return non-nil when Emacs is unable to create images.
 In this scenario (usually caused by running Emacs without a graphical
@@ -183,37 +121,40 @@ Necessary since root icons are not rectangular."
          (h (round (* ,height 1.1818))))
      (setq ,width w ,height h)))
 
-(define-inline treemacs--create-image (file-path)
+(defun treemacs--create-image (file-path)
   "Load image from FILE-PATH and size it based on `treemacs--icon-size'."
-  (inline-letevals (file-path)
-    (inline-quote
-     (let ((height treemacs--icon-size)
-           (width treemacs--icon-size))
-       (when (and (integerp treemacs--icon-size)
-                  (s-starts-with? "root-" ,file-path))
-         (treemacs--root-icon-size-adjust width height))
-       (if (and (integerp treemacs--icon-size) (image-type-available-p 'imagemagick))
-           (create-image ,file-path 'imagemagick nil :ascent 'center :width width :height height)
-         (create-image ,file-path 'png nil :ascent 'center :width width :height height))))))
+  (let ((height treemacs--icon-size)
+        (width treemacs--icon-size))
+    (when (and (integerp treemacs--icon-size)
+               (s-starts-with? "root-" file-path))
+      (treemacs--root-icon-size-adjust width height))
+    (if (and (integerp treemacs--icon-size) (image-type-available-p 'imagemagick))
+        (create-image
+         file-path 'imagemagick nil
+         :ascent 'center
+         :width width
+         :height height
+         :mask 'heuristic)
+      (create-image
+       file-path
+       (intern (treemacs--file-extension (treemacs--filename file-path)))
+       nil
+       :ascent 'center
+       :width width
+       :height height
+       :mask 'heuristic))))
 
-(define-inline treemacs--create-icon-strings (file fallback)
+(defun treemacs--create-icon-strings (file fallback)
   "Create propertized icon strings for a given FILE image and TUI FALLBACK."
-  (inline-letevals (file fallback)
-    (inline-quote
-     (let ((tui-icon ,fallback)
-           (gui-icon
-            (if (treemacs--is-image-creation-impossible?)
-                ,fallback
-              (let* ((img-selected   (treemacs--create-image ,file))
-                     (img-unselected (copy-sequence img-selected)))
-                (nconc img-selected   `(:background ,treemacs--selected-icon-background))
-                (nconc img-unselected `(:background ,treemacs--not-selected-icon-background))
-                (concat (propertize " "
-                                    'display img-unselected
-                                    'img-selected img-selected
-                                    'img-unselected img-unselected)
-                        " ")))))
-       (cons gui-icon tui-icon)))))
+  (let ((tui-icon fallback)
+        (gui-icon
+         (if (treemacs--is-image-creation-impossible?)
+             fallback
+           (concat (propertize
+                    " "
+                    'display (treemacs--create-image file))
+                   " "))))
+    (cons gui-icon tui-icon)))
 
 (defmacro treemacs--splice-icon (icon)
   "Splice the given ICON data depending on whether it is a value or an sexp."
@@ -239,33 +180,43 @@ Necessary since root icons are not rectangular."
   accessible."
   (treemacs-static-assert (or (null icon) (null file))
     "FILE and ICON arguments are mutually exclusive")
-  (-let [ext-list (--map (if (stringp it) (downcase it) it)
-                         (if (symbolp extensions) (symbol-value extensions) extensions))]
-    `(let* ((fallback   ,(if (equal fallback (quote 'same-as-icon))
-                             icon
-                           fallback))
-            (icons-dir  ,(if icons-dir icons-dir `(treemacs-theme->path treemacs--current-theme)))
-            (icon-path  ,(if file `(treemacs-join-path icons-dir ,file) nil))
-            (icon-pair  ,(if file `(treemacs--create-icon-strings icon-path fallback)
-                           `(cons ,(treemacs--splice-icon icon) fallback)))
-            (gui-icons  (treemacs-theme->gui-icons treemacs--current-theme))
-            (tui-icons  (treemacs-theme->tui-icons treemacs--current-theme))
-            (gui-icon   (car icon-pair))
-            (tui-icon   (cdr icon-pair)))
-       ,(unless file
-          `(progn
-             (ignore icon-path)
-             (ignore icons-dir)))
-       ;; prefer to have icons as empty strings with a display property for compatibility
-       ;; in e.g. dired, where an actual text icon would break `dired-goto-file-1'
-       (unless (get-text-property 0 'display gui-icon)
-         (setf gui-icon (propertize " " 'display gui-icon)))
-       ,@(->> (-filter #'symbolp ext-list)
-              (--map `(progn (add-to-list 'treemacs--icon-symbols ',it)
-                             (defvar ,(intern (format "treemacs-icon-%s" it)) nil))))
-       (--each (quote ,ext-list)
-         (ht-set! gui-icons it gui-icon)
-         (ht-set! tui-icons it tui-icon)))))
+  (when (and (consp extensions) (or (symbolp (car extensions))
+                                    (stringp (car extensions))))
+    (setf extensions `(quote (,@extensions))))
+  `(let* ((xs (--map (if (stringp it) (downcase it) it) ,extensions))
+          (fallback   ,(if (equal fallback (quote 'same-as-icon))
+                           icon
+                         fallback))
+          (icons-dir  ,(if icons-dir icons-dir `(treemacs-theme->path treemacs--current-theme)))
+          (icon-path  ,(if file `(treemacs-join-path icons-dir ,file) nil))
+          (icon-pair  ,(if file `(treemacs--create-icon-strings icon-path fallback)
+                         `(cons ,(treemacs--splice-icon icon) fallback)))
+          (gui-icons  (treemacs-theme->gui-icons treemacs--current-theme))
+          (tui-icons  (treemacs-theme->tui-icons treemacs--current-theme))
+          (gui-icon   (car icon-pair))
+          (tui-icon   (cdr icon-pair)))
+     ,(unless file
+        `(progn
+           (ignore icon-path)
+           (ignore icons-dir)))
+     ;; prefer to have icons as empty strings with a display property for compatibility
+     ;; in e.g. dired, where an actual text icon would break `dired-goto-file-1'
+     (unless (get-text-property 0 'display gui-icon)
+       (setf gui-icon (propertize " " 'display gui-icon)))
+     (dolist (ext xs)
+       (when (symbolp ext)
+         (-let [symbol (intern (format "treemacs-icon-%s" ext))]
+           (add-to-list 'treemacs--icon-symbols ext)
+           (set symbol nil))))
+     (--each xs
+       ;; NOTE: Disable creation of GUI svg icons without getting in the way of the rest
+       ;; of the icon creation process. This is good enough a workaround for Emacs versions
+       ;; that don't support svg images for as long as svg icons are a minority.
+       (unless (and ,file
+                    (not (image-type-available-p 'svg))
+                    (string= (treemacs--file-extension ,file) "svg"))
+         (ht-set! gui-icons it gui-icon))
+       (ht-set! tui-icons it tui-icon))))
 
 (treemacs-create-theme "Default"
   :icon-directory (treemacs-join-path treemacs-dir "icons/default")
@@ -282,7 +233,6 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "error.png"             :extensions (error)       :fallback (propertize "• " 'face 'font-lock-string-face))
     (treemacs-create-icon :file "warning.png"           :extensions (warning)     :fallback (propertize "• " 'face 'font-lock-string-face))
     (treemacs-create-icon :file "info.png"              :extensions (info)        :fallback (propertize "• " 'face 'font-lock-string-face))
-    (treemacs-create-icon :file "mail.png"              :extensions (mail)        :fallback " ")
     (treemacs-create-icon :file "bookmark.png"          :extensions (bookmark)    :fallback " ")
     (treemacs-create-icon :file "svgrepo/screen.png"    :extensions (screen)      :fallback " ")
     (treemacs-create-icon :file "svgrepo/house.png"     :extensions (house)       :fallback " ")
@@ -292,11 +242,68 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "svgrepo/close.png"     :extensions (close)       :fallback " ")
     (treemacs-create-icon :file "svgrepo/cal.png"       :extensions (calendar)    :fallback " ")
     (treemacs-create-icon :file "svgrepo/briefcase.png" :extensions (briefcase)   :fallback " ")
+    (treemacs-create-icon :file "svgrepo/mail.png"      :extensions (mail)        :fallback " ")
+    (treemacs-create-icon :file "svgrepo/mail-plus.png" :extensions (mail-plus)   :fallback " ")
+    (treemacs-create-icon :file "svgrepo/inbox.png"     :extensions (inbox)       :fallback " ")
+
+    ;; custom dir icons
+    (treemacs-create-icon :file "svgrepo/dir-src-closed.png"    :extensions ("src-closed")     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "svgrepo/dir-src-open.png"      :extensions ("src-open")       :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "svgrepo/dir-test-closed.png"   :extensions ("test-closed")    :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "svgrepo/dir-test-open.png"     :extensions ("test-open")      :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-binary-closed.png"     :extensions ("bin-closed")     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-binary-open.png"       :extensions ("bin-open")       :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-services-closed.png"   :extensions ("build-closed")   :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-services-open.png"     :extensions ("build-open")     :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "svgrepo/dir-git-closed.png"    :extensions ("git-closed")     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "svgrepo/dir-git-open.png"      :extensions ("git-open")       :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-github-closed.png"     :extensions ("github-closed")  :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-github-open.png"       :extensions ("github-open")    :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-public-closed.png"     :extensions ("public-closed")  :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-public-open.png"       :extensions ("public-open")    :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-private-closed.png"    :extensions ("private-closed") :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon :file "vsc/dir-private-open.png"      :extensions ("private-open")   :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-temp-closed.png" :extensions ("temp-closed" "tmp-closed")
+     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-temp-open.png" :extensions ("temp-open" "tmp-open")
+     :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-docs-closed.png" :extensions ("readme-closed" "docs-closed")
+     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-docs-open.png" :extensions ("readme-open" "docs-open")
+     :fallback (propertize "- " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-images-closed.png" :extensions ("screenshots-closed" "icons-closed")
+     :fallback (propertize "+ " 'face 'treemacs-term-node-face))
+    (treemacs-create-icon
+     :file "vsc/dir-images-open.png" :extensions ("screenshots-open" "icons-open")
+     :fallback (propertize "- " 'face 'treemacs-term-node-face))
+
+    ;; mode icons
+    (treemacs-create-icon :file "emacs.png"          :extensions (emacs-lisp-mode))
+    (treemacs-create-icon :file "git.png"            :extensions (gitignore-mode))
+    (treemacs-create-icon :file "python.png"         :extensions (python-mode))
+    (treemacs-create-icon :file "vsc/make.png"       :extensions (makefile-gmake-mode))
+    (treemacs-create-icon :file "vsc/dir-closed.png" :extensions (dired-mode))
+    (treemacs-create-icon :file "ledger.png"         :extensions (ledger-mode))
+    (treemacs-create-icon :file "vsc/org.png"        :extensions (org-mode org-agenda-mode))
+    (treemacs-create-icon :file "vsc/vim.png"        :extensions (tridactylrc-mode))
+    (treemacs-create-icon :file "conf.png"           :extensions (conf-mode i3wm-config-mode))
+    (treemacs-create-icon :file "conf.png"           :extensions (conf-mode i3wm-config-mode))
+    (treemacs-create-icon :file "shell.png"          :extensions (fish-mode))
+    (treemacs-create-icon :file "svgrepo/list.png"   :extensions (elfeed-search-mode))
+    (treemacs-create-icon :file "vsc/deps.png"       :extensions (cask-mode))
+    (treemacs-create-icon :file "yaml.png"           :extensions (yaml-mode))
+    (treemacs-create-icon :file "css.png"            :extensions (css-mode))
+    (treemacs-create-icon :file "markdown.png"       :extensions (markdown-mode))
 
     ;; file icons
     (treemacs-create-icon :file "txt.png"           :extensions (fallback))
-    (treemacs-create-icon :file "emacs.png"         :extensions ("el" "elc"))
-    (treemacs-create-icon :file "ledger.png"        :extensions ("ledger"))
+    (treemacs-create-icon :file "emacs.png"         :extensions ("el" "elc" "eln"))
+    (treemacs-create-icon :file "ledger.png"        :extensions ("ledger" "beancount"))
     (treemacs-create-icon :file "yaml.png"          :extensions ("yml" "yaml" "travis.yml"))
     (treemacs-create-icon
      :file "shell.png"
@@ -312,10 +319,10 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "asciidoc.png"      :extensions ("adoc" "asciidoc"))
     (treemacs-create-icon :file "rust.png"          :extensions ("rs"))
     (treemacs-create-icon :file "image.png"         :extensions ("jpg" "jpeg" "bmp" "svg" "png" "xpm" "gif"))
-    (treemacs-create-icon :file "clojure.png"       :extensions ("clj" "cljs" "cljc"))
+    (treemacs-create-icon :file "clojure.png"       :extensions ("clj" "cljs" "cljc" "cljd" "edn"))
     (treemacs-create-icon :file "ts.png"            :extensions ("ts" "tsx"))
     (treemacs-create-icon :file "vue.png"           :extensions ("vue"))
-    (treemacs-create-icon :file "css.png"           :extensions ("css"))
+    (treemacs-create-icon :file "css.png"           :extensions ("css" "rasi"))
     (treemacs-create-icon :file "conf.png"          :extensions ("properties" "conf" "config" "cfg" "ini" "xdefaults" "xresources" "terminalrc" "ledgerrc"))
     (treemacs-create-icon :file "html.png"          :extensions ("html" "htm"))
     (treemacs-create-icon :file "git.png"           :extensions ("git" "gitignore" "gitconfig" "gitmodules" "gitattributes"))
@@ -323,7 +330,7 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "jar.png"           :extensions ("jar"))
     (treemacs-create-icon :file "kotlin.png"        :extensions ("kt"))
     (treemacs-create-icon :file "scala.png"         :extensions ("scala"))
-    (treemacs-create-icon :file "gradle.png"        :extensions ("gradle"))
+    (treemacs-create-icon :file "gradle.png"        :extensions ("gradle" "kts" "build.gradle.kts" "settings.gradle.kts"))
     (treemacs-create-icon :file "sbt.png"           :extensions ("sbt"))
     (treemacs-create-icon :file "go.png"            :extensions ("go"))
     (treemacs-create-icon :file "systemd.png"       :extensions ("service" "timer"))
@@ -334,7 +341,7 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "json.png"          :extensions ("json"))
     (treemacs-create-icon :file "julia.png"         :extensions ("jl"))
     (treemacs-create-icon :file "elx.png"           :extensions ("ex"))
-    (treemacs-create-icon :file "elx-light.png"     :extensions ("exs" "eex" "leex"))
+    (treemacs-create-icon :file "elx-light.png"     :extensions ("exs" "eex" "leex" "heex"))
     (treemacs-create-icon :file "ocaml.png"         :extensions ("ml" "mli" "merlin" "ocaml"))
     (treemacs-create-icon :file "direnv.png"        :extensions ("envrc"))
     (treemacs-create-icon :file "puppet.png"        :extensions ("pp"))
@@ -371,8 +378,8 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "vsc/asm.png"       :extensions ("asm" "arm"))
     (treemacs-create-icon :file "vsc/autohk.png"    :extensions ("ahk"))
     (treemacs-create-icon :file "vsc/babel.png"     :extensions ("babelrc" "babelignore" "babelrc.js" "babelrc.json" "babel.config.js"))
-    (treemacs-create-icon :file "vsc/bat.png"       :extensions ("bat"))
-    (treemacs-create-icon :file "vsc/binary.png"    :extensions ("exe" "dll" "obj" "so" "o"))
+    (treemacs-create-icon :file "vsc/bat.png"       :extensions ("bat" "exe" "gradlew"))
+    (treemacs-create-icon :file "vsc/binary.png"    :extensions ("dll" "obj" "so" "o"))
     (treemacs-create-icon :file "vsc/bazel.png"     :extensions ("bazelrc" "bazel"))
     (treemacs-create-icon :file "vsc/bower.png"     :extensions ("bowerrc" "bower.json"))
     (treemacs-create-icon :file "vsc/bundler.png"   :extensions ("gemfile" "gemfile.lock"))
@@ -383,7 +390,7 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "vsc/cobol.png"     :extensions ("cobol"))
     (treemacs-create-icon :file "vsc/cfscript.png"  :extensions ("coffeescript"))
     (treemacs-create-icon :file "vsc/cpp.png"       :extensions ("cpp" "cxx" "tpp" "cc"))
-    (treemacs-create-icon :file "vsc/cpph.png"      :extensions ("hpp" "hh"))
+    (treemacs-create-icon :file "vsc/cpph.png"      :extensions ("hpp" "hxx" "hh"))
     (treemacs-create-icon :file "vsc/cucumber.png"  :extensions ("feature"))
     (treemacs-create-icon :file "vsc/cython.png"    :extensions ("cython"))
     (treemacs-create-icon :file "vsc/delphi.png"    :extensions ("pascal" "objectpascal"))
@@ -442,10 +449,12 @@ Necessary since root icons are not rectangular."
     (treemacs-create-icon :file "vsc/vim.png"       :extensions ("vimrc" "tridactylrc" "vimperatorrc" "ideavimrc" "vrapperrc"))
     (treemacs-create-icon :file "vsc/deps.png"      :extensions ("cask"))
     (treemacs-create-icon :file "vsc/r.png"         :extensions ("r"))
+    (treemacs-create-icon :file "vsc/terraform.png" :extensions ("tf" "terraform"))
     (treemacs-create-icon :file "vsc/reason.png"    :extensions ("re" "rei"))))
 
 (define-inline treemacs-icon-for-file (file)
   "Retrieve an icon for FILE from `treemacs-icons' based on its extension.
+Works only with files, not directories.
 Uses `treemacs-icon-fallback' as fallback."
   (declare (side-effect-free t))
   (inline-letevals (file)
@@ -456,12 +465,42 @@ Uses `treemacs-icon-fallback' as fallback."
                    (treemacs--file-extension file-downcased)
                    (with-no-warnings treemacs-icon-fallback)))))))
 
+(define-inline treemacs-icon-for-mode (mode)
+  "Retrieve an icon for major MODE from `treemacs-icons'.
+Uses `treemacs-icon-fallback' as fallback."
+  (declare (side-effect-free t))
+  (inline-letevals (mode)
+    (inline-quote
+     (ht-get treemacs-icons ,mode (with-no-warnings treemacs-icon-fallback)))))
+
+(define-inline treemacs-icon-for-dir (dir state)
+  "Retrieve an icon for DIR from `treemacs-icons' in given STATE.
+STATE must be either `open' or `closed'.
+
+Works only with directories, not files.
+Uses the `dir-open' and `dir-closed' icons as fallback."
+  (declare (side-effect-free t))
+  (inline-letevals (dir state)
+    (inline-quote
+     (let ((name-downcased (-> ,dir (treemacs--filename) (downcase))))
+       (when (eq ?. (aref name-downcased 0))
+         (setf name-downcased (substring name-downcased 1)))
+       (pcase-exhaustive ,state
+         (`open
+          (let ((name (format "%s-%s" name-downcased "open")))
+            (or (ht-get treemacs-icons name)
+                (ht-get treemacs-icons 'dir-open))))
+         (`closed
+          (let ((name (format "%s-%s" name-downcased "closed")))
+            (or (ht-get treemacs-icons name)
+                (ht-get treemacs-icons 'dir-closed)))))))))
+
 ;;;###autoload
 (defun treemacs-resize-icons (size)
   "Resize the current theme's icons to the given SIZE.
 
-If SIZE is 'nil' the icons are not resized and will retain their default size of
-22 pixels.
+If SIZE is \\='nil' the icons are not resized and will retain their default size
+of 22 pixels.
 
 There is only one size, the icons are square and the aspect ratio will be
 preserved when resizing them therefore width and height are the same.
@@ -478,17 +517,14 @@ png are changed."
       (treemacs-log-failure "Icons cannot be resized without image transforms or imagemagick support.")
     (setq treemacs--icon-size size)
     (treemacs--maphash (treemacs-theme->gui-icons treemacs--current-theme) (_ icon)
-      (let ((display        (get-text-property 0 'display icon))
-            (img-selected   (get-text-property 0 'img-selected icon))
-            (img-unselected (get-text-property 0 'img-unselected icon))
-            (width          treemacs--icon-size)
-            (height         treemacs--icon-size))
+      (let ((display (get-text-property 0 'display icon))
+            (width   treemacs--icon-size)
+            (height  treemacs--icon-size))
         (when (eq 'image (car-safe display))
           (when (s-ends-with? "root.png" (plist-get (cdr display) :file))
             (treemacs--root-icon-size-adjust width height))
-          (dolist (property (list display img-selected img-unselected))
-            (plist-put (cdr property) :height height)
-            (plist-put (cdr property) :width width)))))))
+          (plist-put (cdr display) :height height)
+          (plist-put (cdr display) :width width))))))
 
 (defun treemacs--select-icon-set ()
   "Select the right set of icons for the current buffer.
@@ -523,7 +559,7 @@ Return the fallback icons if TUI is non-nil."
 Note that treemacs has a very loose definition of what constitutes a file
 extension - it's either everything past the last period, or just the file's full
 name if there is no period.  This makes it possible to match file names like
-'.gitignore' and 'Makefile'.
+\\='.gitignore' and \\='Makefile'.
 
 Additionally FILE-EXTENSIONS are also not case sensitive and will be stored in a
 down-cased state."
@@ -552,21 +588,20 @@ should be used for."
 (defun treemacs-map-icons-with-auto-mode-alist (extensions mode-icon-alist)
   "Remaps icons for EXTENSIONS according to `auto-mode-alist'.
 EXTENSIONS should be a list of file extensions such that they match the regex
-stored in `auto-mode-alist', for example '\(\".cc\"\).
+stored in `auto-mode-alist', for example \\='(\".cc\").
 MODE-ICON-ALIST is an alist that maps which mode from `auto-mode-alist' should
 be assigned which treemacs icon, for example
-'\(\(c-mode . treemacs-icon-c\)
-  \(c++-mode . treemacs-icon-cpp\)\)"
+`((c-mode . ,(treemacs-get-icon-value \"c\"))
+  (c++-mode . ,(treemacs-get-icon-value \"cpp\")))"
   (dolist (extension extensions)
     (-when-let* ((mode (cdr (--first (s-matches? (car it) extension) auto-mode-alist)))
                  (icon (cdr (assq mode mode-icon-alist))))
-      (treemacs-log "Map %s to %s" extension (symbol-name icon))
       (ht-set! (treemacs-theme->gui-icons treemacs--current-theme)
                (substring extension 1)
-               (symbol-value icon)))))
+               icon))))
 
 (treemacs-only-during-init
-  (treemacs-load-theme "Default"))
+ (treemacs-load-theme "Default"))
 
 (provide 'treemacs-icons)
 
