@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 (require 'treemacs-customization)
 (require 'treemacs-workspaces)
 (require 'treemacs-persistence)
-(require 'treemacs-extensions)
 (require 'treemacs-logging)
 
 (eval-when-compile
@@ -41,7 +40,9 @@
   (require 'treemacs-macros))
 
 (autoload 'ansi-color-apply-on-region "ansi-color")
-(autoload 'aw-select "ace-window")
+
+(treemacs-import-functions-from "ace-window"
+  ace-select-window)
 
 (treemacs-import-functions-from "cfrs"
   cfrs-read)
@@ -122,7 +123,8 @@ them instead."
    :on-tag-node-open    (treemacs--collapse-tag-node btn arg)
    :on-tag-node-closed  (treemacs--expand-tag-node btn arg)
    :on-tag-node-leaf    (progn (other-window 1) (treemacs--goto-tag btn))
-   :on-nil              (treemacs-pulse-on-failure "There is nothing to do here.")))
+   :on-nil              (treemacs-pulse-on-failure "There is nothing to do here.")
+   :fallback            (treemacs-TAB-action)))
 
 (defun treemacs-toggle-node-prefer-tag-visit (&optional arg)
   "Same as `treemacs-toggle-node' but will visit a tag node in some conditions.
@@ -133,22 +135,24 @@ conditions:
    ends in \" definition*\". This indicates the section is the parent element in
    a nested class/function definition and can be moved to.
  * Tags belong to a .org file and the tag section element possesses a
-   'org-imenu-marker text property. This indicates that the section is a
+   \\='org-imenu-marker text property. This indicates that the section is a
    headline with further org elements below it.
 
 The prefix argument ARG is treated the same way as with `treemacs-toggle-node'."
   (interactive)
-  (treemacs-do-for-button-state
-   :on-root-node-open   (treemacs--collapse-root-node btn arg)
-   :on-root-node-closed (treemacs--expand-root-node btn)
-   :on-dir-node-open    (treemacs--collapse-dir-node btn arg)
-   :on-dir-node-closed  (treemacs--expand-dir-node btn :recursive arg)
-   :on-file-node-open   (treemacs--collapse-file-node btn arg)
-   :on-file-node-closed (treemacs--expand-file-node btn arg)
-   :on-tag-node-open    (treemacs--visit-or-expand/collapse-tag-node btn arg t)
-   :on-tag-node-closed  (treemacs--visit-or-expand/collapse-tag-node btn arg t)
-   :on-tag-node-leaf    (progn (other-window 1) (treemacs--goto-tag btn))
-   :on-nil              (treemacs-pulse-on-failure "There is nothing to do here.")))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs-do-for-button-state
+    :on-root-node-open   (treemacs--collapse-root-node btn arg)
+    :on-root-node-closed (treemacs--expand-root-node btn)
+    :on-dir-node-open    (treemacs--collapse-dir-node btn arg)
+    :on-dir-node-closed  (treemacs--expand-dir-node btn :recursive arg)
+    :on-file-node-open   (treemacs--collapse-file-node btn arg)
+    :on-file-node-closed (treemacs--expand-file-node btn arg)
+    :on-tag-node-open    (treemacs--visit-or-expand/collapse-tag-node btn arg t)
+    :on-tag-node-closed  (treemacs--visit-or-expand/collapse-tag-node btn arg t)
+    :on-tag-node-leaf    (progn (other-window 1) (treemacs--goto-tag btn))
+    :on-nil              (treemacs-pulse-on-failure "There is nothing to do here."))))
 
 (defun treemacs-TAB-action (&optional arg)
   "Run the appropriate TAB action for the current node.
@@ -197,28 +201,32 @@ ARG is optional and only available so this function can be used as an action."
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :split-function #'split-window-vertically
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :split-function #'split-window-vertically
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-horizontal-split (&optional arg)
   "Open current file or tag by horizontally splitting `next-window'.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :split-function #'split-window-horizontally
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :split-function #'split-window-horizontally
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-close-treemacs (&optional _)
   "Open current node without and close treemacs.
@@ -235,75 +243,85 @@ window then that window will be selected instead.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :ensure-window-split t
-   :window  (-some-> btn (treemacs--nearest-path) (get-file-buffer) (get-buffer-window))
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :ensure-window-split t
+    :window  (-some-> btn (treemacs--nearest-path) (get-file-buffer) (get-buffer-window))
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-ace (&optional arg)
   "Open current file or tag in window selected by `ace-window'.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :window (aw-select "Select window")
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :ensure-window-split t
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :window (ace-select-window)
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :ensure-window-split t
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-in-most-recently-used-window (&optional arg)
   "Open current file or tag in window selected by `get-mru-window'.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :window (get-mru-window (selected-frame) nil :not-selected)
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :ensure-window-split t
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :window (get-mru-window (selected-frame) nil :not-selected)
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :ensure-window-split t
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-ace-horizontal-split (&optional arg)
   "Open current file by horizontally splitting window selected by `ace-window'.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :split-function #'split-window-horizontally
-   :window (aw-select "Select window")
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :split-function #'split-window-horizontally
+    :window (ace-select-window)
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-ace-vertical-split (&optional arg)
   "Open current file by vertically splitting window selected by `ace-window'.
 Stay in the current window with a single prefix argument ARG, or close the
 treemacs window with a double prefix argument."
   (interactive "P")
-  (treemacs--execute-button-action
-   :split-function #'split-window-vertically
-   :window (aw-select "Select window")
-   :file-action (find-file (treemacs-safe-button-get btn :path))
-   :dir-action (dired (treemacs-safe-button-get btn :path))
-   :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
-   :tag-action (treemacs--goto-tag btn)
-   :window-arg arg
-   :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here."))
+  (run-hook-with-args
+   'treemacs-after-visit-functions
+   (treemacs--execute-button-action
+    :split-function #'split-window-vertically
+    :window (ace-select-window)
+    :file-action (find-file (treemacs-safe-button-get btn :path))
+    :dir-action (dired (treemacs-safe-button-get btn :path))
+    :tag-section-action (treemacs--visit-or-expand/collapse-tag-node btn arg nil)
+    :tag-action (treemacs--goto-tag btn)
+    :window-arg arg
+    :no-match-explanation "Node is neither a file, a directory or a tag - nothing to do here.")))
 
 (defun treemacs-visit-node-default (&optional arg)
   "Run `treemacs-default-visit-action' for the current button.
@@ -353,7 +371,8 @@ In the default configuration this usually means to close the content of the
 currently selected node.  A potential prefix ARG is passed on to the executed
 action, if possible.
 
-This function's exact configuration is stored in `treemacs-COLLAPSE-actions-config'."
+This function's exact configuration is stored in
+`treemacs-COLLAPSE-actions-config'."
   (interactive "P")
   (-when-let (state (treemacs--prop-at-point :state))
     (--if-let (cdr (assq state treemacs-COLLAPSE-actions-config))
@@ -508,7 +527,8 @@ With a prefix ARG substract the increment value multiple times."
        "Path at point is not a file.")
      (when (file-directory-p path)
        (setf path (treemacs--add-trailing-slash path)))
-     (-let [copied (-> path (file-relative-name (treemacs-project->path project)) (kill-new))]
+     (-let [copied (-> path (file-relative-name (treemacs-project->path project)))]
+       (kill-new copied)
        (treemacs-pulse-on-success "Copied relative path: %s" (propertize copied 'face 'font-lock-string-face))))))
 
 (defun treemacs-copy-project-path-at-point ()
@@ -520,9 +540,41 @@ With a prefix ARG substract the increment value multiple times."
        "There is nothing to copy here")
      (treemacs-error-return-if (not (stringp (treemacs-project->path project)))
        "Project at point is not a file.")
-    (-let [copied (-> project (treemacs-project->path) (kill-new))]
+    (-let [copied (-> project (treemacs-project->path))]
+      (kill-new copied)
       (treemacs-pulse-on-success "Copied project path: %s" (propertize copied 'face 'font-lock-string-face))))))
 
+(defun treemacs-copy-filename-at-point ()
+  "Copy the filename of the node at point."
+  (interactive)
+  (treemacs-block
+   (-let [path (treemacs--prop-at-point :path)]
+     (treemacs-error-return-if (null path)
+       "There is nothing to copy here")
+     (treemacs-error-return-if (not (stringp path))
+       "Path at point is not a file.")
+     (let ((filename (file-name-nondirectory path)))
+       (kill-new filename)
+       (treemacs-pulse-on-success "Copied filename: %s" (propertize filename 'face 'font-lock-string-face))))))
+
+(defun treemacs-paste-dir-at-point-to-minibuffer ()
+  "Paste the directory at point into the minibuffer.
+This is used by the \"Paste here\" mouse menu button, which assumes that we are
+running `treemacs--copy-or-move', so that pasting this path into the minibuffer
+allows us to copy/move the previously-selected file into the path at point."
+  (interactive)
+  (treemacs-block
+   (treemacs-error-return-if (not (active-minibuffer-window))
+     "Minibuffer is not active")
+   (let* ((path-at-point (treemacs--prop-at-point :path))
+          (dir (if (file-directory-p path-at-point)
+                   path-at-point
+                 (file-name-directory path-at-point))))
+     (select-window (active-minibuffer-window))
+     (delete-region (minibuffer-prompt-end) (point-max))
+     (insert dir))
+   (message "Copied from treemacs")))
+
 (defun treemacs-delete-other-windows ()
   "Same as `delete-other-windows', but will not delete the treemacs window.
 If this command is run when the treemacs window is selected `next-window' will
@@ -537,7 +589,7 @@ also not be deleted."
       ;; that we shouldn't prevent from running, so we just restore treemacs instead of preventing
       ;; it from being deleted
       ;; 'no-delete-other-windows could be used instead, but it's only available for emacs 26
-      (when w
+      (when (and w (not (equal 'visible (treemacs-current-visibility))))
         (treemacs--select-not-visible-window)))))
 
 (defun treemacs-temp-resort-root (&optional sort-method)
@@ -704,10 +756,9 @@ For slower scrolling see `treemacs-previous-line-other-window'"
            (treemacs-return-if (string-equal old-name new-name)
              (treemacs-pulse-on-failure "The new name is the same as the old name."))
            (setf (treemacs-project->name project) new-name)
-           (treemacs--forget-last-highlight)
            ;; after renaming, delete and redisplay the project
            (goto-char (treemacs-button-end project-btn))
-           (delete-region (point-at-bol) (point-at-eol))
+           (delete-region (line-beginning-position) (line-end-position))
            (treemacs--add-root-element project)
            (when (eq state 'root-node-open)
              (treemacs--collapse-root-node (treemacs-project->position project))
@@ -747,7 +798,7 @@ auto-selected name already exists."
        (propertize (treemacs-project->path duplicate) 'face 'font-lock-string-face)))
     (`(includes-project ,project)
      (goto-char (treemacs-project->position project))
-     (treemacs-pulse-on-failure "Project '%s' is included in '%s'. Projects May not overlap."
+     (treemacs-pulse-on-failure "Project '%s' is included in '%s'. Projects may not overlap."
        (propertize (treemacs-project->name project) 'face 'font-lock-type-face)
        (propertize path 'face 'font-lock-string-face)))
     (`(duplicate-name ,duplicate)
@@ -871,8 +922,10 @@ workspaces."
   (interactive)
   (treemacs-unless-let (btn (treemacs-current-button))
       (treemacs-log-failure "There is nothing to refresh.")
-    (treemacs-without-recenter
-     (treemacs--do-refresh (current-buffer) (treemacs-project-of-node btn)))))
+    (-let [project (treemacs-project-of-node btn)]
+      (treemacs-without-recenter
+       (treemacs--do-refresh (current-buffer) project))
+      (run-hook-with-args 'treemacs-post-project-refresh-functions project))))
 
 (defun treemacs-collapse-project (&optional arg)
   "Close the project at point.
@@ -882,7 +935,6 @@ With a prefix ARG also forget about all the nodes opened in the project."
       (treemacs-pulse-on-failure "There is nothing to close here.")
     (-let [btn (treemacs-project->position project)]
       (when (treemacs-is-node-expanded? btn)
-        (treemacs--forget-last-highlight)
         (goto-char btn)
         (treemacs--collapse-root-node btn arg)
         (treemacs--maybe-recenter 'on-distance)))
@@ -890,17 +942,18 @@ With a prefix ARG also forget about all the nodes opened in the project."
 
 (defun treemacs-collapse-all-projects (&optional arg)
   "Collapses all projects.
-With a prefix ARG also forget about all the nodes opened in the projects."
+With a prefix ARG remember which nodes were expanded."
   (interactive "P")
-  (save-excursion
-    (treemacs--forget-last-highlight)
-    (dolist (project (treemacs-workspace->projects (treemacs-current-workspace)))
-      (-when-let (pos (treemacs-project->position project))
-        (when (eq 'root-node-open (treemacs-button-get pos :state))
-          (goto-char pos)
-          (treemacs--collapse-root-node pos arg)))))
-  (treemacs--maybe-recenter 'on-distance)
-  (treemacs-pulse-on-success "Collapsed all projects"))
+  (-when-let (buffer (treemacs-get-local-buffer))
+    (with-current-buffer buffer
+      (save-excursion
+        (dolist (project (treemacs-workspace->projects (treemacs-current-workspace)))
+          (-when-let (pos (treemacs-project->position project))
+            (when (eq 'root-node-open (treemacs-button-get pos :state))
+              (goto-char pos)
+              (treemacs--collapse-root-node pos (not arg))))))
+      (treemacs--maybe-recenter 'on-distance)
+      (treemacs-pulse-on-success "Collapsed all projects"))))
 
 (defun treemacs-collapse-other-projects (&optional arg)
   "Collapses all projects except the project at point.
@@ -1073,7 +1126,7 @@ Only works with a single project in the workspace."
          treemacs-persist-file
          nil :silent)
         (treemacs--restore)
-        (-if-let (ws (treemacs--select-workspace-by-name
+        (-if-let (ws (treemacs--find-workspace-by-name
                       (treemacs-workspace->name (treemacs-current-workspace))))
             (setf (treemacs-current-workspace) ws)
           (treemacs--find-workspace))
@@ -1096,7 +1149,6 @@ Prefix ARG will be passed on to the closing function
   (-if-let* ((btn (treemacs-current-button))
              (parent (button-get btn :parent)))
       (progn
-        (treemacs--forget-last-highlight)
         (goto-char parent)
         (treemacs-toggle-node arg)
         (treemacs--evade-image))
@@ -1261,7 +1313,7 @@ visible."
     (save-excursion
       (goto-char (point-min))
       (while (= 0 (forward-line 1))
-        (-let [new-len (- (point-at-eol) (point-at-bol))]
+        (-let [new-len (- (line-end-position) (line-beginning-position))]
           (when (> new-len longest)
             (setf longest new-len
                   depth (treemacs--prop-at-point :depth))))))
@@ -1288,6 +1340,53 @@ Specifically this will toggle treemacs' width between
     (put 'treemacs-extra-wide-toggle :toggle-on t)
     (treemacs-log "Switched to extra width display")))
 
+(defun treemacs-next-workspace (&optional arg)
+  "Switch to the next workspace.
+With a prefix ARG switch to the previous workspace instead."
+  (interactive)
+  (treemacs-block
+   (treemacs-error-return-if (= 1 (length treemacs--workspaces))
+     "There is only 1 workspace.")
+   (let* ((ws (treemacs-current-workspace))
+          (ws-count (length treemacs--workspaces))
+          (idx (--find-index (eq it ws) treemacs--workspaces))
+          (new-idx (% (+ ws-count (if arg (1- idx) (1+ idx))) ws-count))
+          (new-ws (nth new-idx treemacs--workspaces)))
+     (treemacs-do-switch-workspace new-ws)
+     (treemacs-pulse-on-success "Switched to workspace '%s'"
+       (propertize (treemacs-workspace->name new-ws)
+                   'face 'font-lock-string-face)))))
+
+(defun treemacs-create-workspace-from-project (&optional arg)
+  "Create (and switch to) a workspace containing only the current project.
+
+By default uses the project at point in the treemacs buffer.  If there is no
+treemacs buffer, then the project of the current file is used instead.  With a
+prefix ARG it is also possible to interactively select the project."
+  (interactive "P")
+  (treemacs-block
+    (-let [project nil]
+      (if (eq t treemacs--in-this-buffer)
+          (setf project (treemacs-project-of-node (treemacs-current-button)))
+        (setf project (treemacs--find-project-for-buffer (buffer-file-name (current-buffer))))
+        (treemacs-select-window))
+      (when (or arg (null project))
+        (setf project (treemacs--select-project-by-name))
+        (treemacs-return-if (null project)))
+      (let* ((ws-name (treemacs-project->name project))
+             (new-ws (treemacs--find-workspace-by-name ws-name)))
+        (if new-ws
+            (setf (treemacs-workspace->projects new-ws) (list project))
+          (-let [ws-create-result (treemacs-do-create-workspace ws-name)]
+            (treemacs-error-return-if (not (equal 'success (car ws-create-result)))
+              "Something went wrong when creating a new workspace: %s" ws-create-result)
+            (setf new-ws (cdr ws-create-result))
+            (setf (treemacs-workspace->projects new-ws) (list project))
+            (treemacs--persist)))
+        (treemacs-do-switch-workspace new-ws)
+        (treemacs-pulse-on-success "Switched to project workspace '%s'"
+          (propertize ws-name 'face 'font-lock-type-face))))))
+
 (defun treemacs-icon-catalogue ()
   "Showcase a catalogue of all treemacs themes and their icons."
   (interactive)
