@@ -1,6 +1,6 @@
 ;;; treemacs-mode.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
   treemacs-advanced-helpful-hydra)
 
 (treemacs-import-functions-from "treemacs-tags"
-  treemacs--create-imenu-index-functione)
+  treemacs--create-imenu-index-function)
 
 (defvar bookmark-make-record-function)
 
@@ -102,6 +102,7 @@ Will be set by `treemacs--post-command'.")
     (define-key map (kbd "s")     'treemacs-switch-workspace)
     (define-key map (kbd "e")     'treemacs-edit-workspaces)
     (define-key map (kbd "f")     'treemacs-set-fallback-workspace)
+    (define-key map (kbd "n")     'treemacs-next-workspace)
     map)
   "Keymap for workspace-related commands in `treemacs-mode'.")
 
@@ -129,6 +130,8 @@ Will be set by `treemacs--post-command'.")
     (define-key map (kbd "f")        'treemacs-follow-mode)
     (define-key map (kbd "a")        'treemacs-filewatch-mode)
     (define-key map (kbd "n")        'treemacs-indent-guide-mode)
+    (define-key map (kbd "c")        'treemacs-indicate-top-scroll-mode)
+    (define-key map (kbd "d")        'treemacs-git-commit-diff-mode)
     map)
   "Keymap for commands that toggle state in `treemacs-mode'.")
 
@@ -137,7 +140,9 @@ Will be set by `treemacs--post-command'.")
     (define-key map (kbd "a")        'treemacs-copy-absolute-path-at-point)
     (define-key map (kbd "r")        'treemacs-copy-relative-path-at-point)
     (define-key map (kbd "p")        'treemacs-copy-project-path-at-point)
+    (define-key map (kbd "n")        'treemacs-copy-filename-at-point)
     (define-key map (kbd "f")        'treemacs-copy-file)
+    (define-key map (kbd "v")        'treemacs-paste-dir-at-point-to-minibuffer)
     map)
   "Keymap for copy commands in `treemacs-mode'.")
 
@@ -199,6 +204,9 @@ Will be set by `treemacs--post-command'.")
     (define-key map (kbd "C")         'treemacs-cleanup-litter)
     (define-key map (kbd "=")         'treemacs-fit-window-width)
     (define-key map (kbd "W")         'treemacs-extra-wide-toggle)
+    (define-key map (kbd "M-m")       'treemacs-bulk-file-actions)
+    (unless (window-system)
+      (define-key map [C-i] 'treemacs-TAB-action))
     map)
   "Keymap for `treemacs-mode'.")
 
@@ -216,15 +224,24 @@ Will be set by `treemacs--post-command'.")
                              major-mode)
                 nil)
                '("%e" (:eval (spaceline-ml-treemacs))))
-              ((memq 'moody-mode-line-buffer-identification
-                     (default-value 'mode-line-format))
+              ((and (listp (default-value 'mode-line-format))
+                    (member 'moody-mode-line-buffer-identification
+                            (default-value 'mode-line-format)))
                '(:eval (moody-tab " Treemacs " 10 'down)))
-              ((and (fboundp 'doom-modeline)
-                    (fboundp 'doom-modeline-def-modeline))
-               (doom-modeline-def-modeline 'treemacs '(bar " " major-mode))
-               (doom-modeline 'treemacs))
+              ((featurep 'doom-modeline)
+               (with-no-warnings
+                 (eval
+                  '(progn
+                     (require 'doom-modeline)
+                     (doom-modeline-def-segment treemacs-workspace-name
+                       "Display treemacs."
+                       (propertize (format " %s " (treemacs-workspace->name (treemacs-current-workspace)))
+                                   'face (doom-modeline-face 'doom-modeline-buffer-minor-mode)))
+                     (doom-modeline-def-modeline 'treemacs '(bar " " major-mode) '(treemacs-workspace-name))
+                     (doom-modeline 'treemacs)))))
               (t
-               '(" Treemacs ")))))
+               '(:eval (format " Treemacs: %s"
+                               (treemacs-workspace->name (treemacs-current-workspace))))))))
 
 (defun treemacs--post-command ()
   "Set the default directory to the nearest directory of the current node.
@@ -232,6 +249,18 @@ If there is no node at point use \"~/\" instead.
 Also skip hidden buttons (as employed by variadic extensions).
 
 Used as a post command hook."
+  (let ((newline-char 10)
+        (point-max (point-max)))
+    (unless (equal newline-char (char-before point-max))
+      (treemacs-with-writable-buffer
+       (save-excursion
+         (goto-char point-max)
+         (insert newline-char)
+         ;; make sure that the projects-end marker keeps pointing at
+         ;; the end of the last project button
+         (when (and (eq t treemacs--in-this-buffer)
+                    (equal (point) (marker-position (treemacs--projects-end))))
+           (move-marker (treemacs--projects-end) (1- (point))))))))
   (-when-let (btn (treemacs-current-button))
     (when (treemacs-button-get btn 'invisible)
       (treemacs-next-line 1))
@@ -240,17 +269,56 @@ Used as a post command hook."
                          (treemacs--nearest-path btn))))
         (when (and (treemacs-project->is-readable? project)
                    (file-readable-p path))
-          (setq treemacs--eldoc-msg path
+          (setf treemacs--eldoc-msg (treemacs--get-eldoc-message path)
                 default-directory (treemacs--add-trailing-slash
                                    (if (file-directory-p path) path (file-name-directory path)))))
-      (setq treemacs--eldoc-msg nil
-            default-directory "~/"))))
+      (setf treemacs--eldoc-msg nil)
+      (when (eq t treemacs--in-this-buffer)
+        (setf default-directory "~/")))))
+
+(defun treemacs--get-eldoc-message (path)
+  "Set the eldoc message for given PATH.
+Message will be either just the path, or the path plus meta info like file size,
+depending on the value of `treemacs-eldoc-display'."
+  (pcase treemacs-eldoc-display
+    ('detailed
+     (-let [attr (file-attributes path)]
+       (format "%s -- %s: %s %s: %s %s: %s"
+               (propertize path 'face 'font-lock-string-face)
+               (propertize "Size" 'face 'font-lock-keyword-face)
+               (propertize
+                (treemacs--human-readable-bytes (file-attribute-size attr))
+                'face 'font-lock-type-face)
+               (propertize "Last Modified" 'face 'font-lock-keyword-face)
+               (propertize
+                (format-time-string "%F %T" (file-attribute-modification-time attr))
+                'face 'font-lock-type-face)
+               (propertize "Permissions" 'face 'font-lock-keyword-face)
+               (propertize
+                (file-attribute-modes attr)
+                'face 'font-lock-type-face))))
+    ('simple (propertize path 'face 'font-lock-string-face))
+    (_       (propertize path 'face 'font-lock-string-face))))
+
+(define-inline treemacs--human-readable-bytes (bytes)
+  "Return a human-readable string version of BYTES."
+  (declare (pure t) (side-effect-free t))
+  (inline-letevals (bytes)
+    (inline-quote
+     (cl-loop with result = (cons "B" ,bytes)
+              for i in '("k" "M" "G" "T" "P" "E" "Z" "Y")
+              while (>= (cdr result) 1024.0)
+              do (setf result (cons i (/ (cdr result) 1024.0)))
+              finally return
+              (pcase (car result)
+                ("B" (format "%sb" ,bytes))
+                (_ (format "%.1f%s" (cdr result) (car result))))))))
 
 (defun treemacs--eldoc-function ()
   "Treemacs' implementation of `eldoc-documentation-function'.
 Will simply return `treemacs--eldoc-msg'."
   (when (and treemacs-eldoc-display treemacs--eldoc-msg)
-    (propertize treemacs--eldoc-msg 'face 'font-lock-string-face)))
+    treemacs--eldoc-msg))
 
 ;;;###autoload
 (define-derived-mode treemacs-mode special-mode "Treemacs"
@@ -270,7 +338,7 @@ Will simply return `treemacs--eldoc-msg'."
       (setq evil-treemacs-state-cursor
             (if treemacs-show-cursor
                 evil-motion-state-cursor
-              '(hbar . 0)))))
+              (lambda () (setq cursor-type nil))))))
 
   ;; higher fuzz value makes it less likely to start a mouse drag
   ;; and make a switch to visual state
@@ -280,6 +348,9 @@ Will simply return `treemacs--eldoc-msg'."
   (setq-local eldoc-documentation-function #'treemacs--eldoc-function)
   (setq-local eldoc-message-commands treemacs--eldoc-obarray)
   (setq-local imenu-create-index-function #'treemacs--create-imenu-index-function)
+  (when (boundp 'context-menu-functions)
+    (setq-local context-menu-functions nil))
+
   ;; integrate with bookmark.el
   (setq-local bookmark-make-record-function #'treemacs--make-bookmark-record)
   (electric-indent-local-mode -1)
@@ -305,10 +376,9 @@ Will simply return `treemacs--eldoc-msg'."
   ;; alongside other window layout chaning commands that might delete it again
   (set-window-parameter (selected-window) 'no-delete-other-windows treemacs-no-delete-other-windows)
 
-  (when treemacs-window-background-color
-    (face-remap-add-relative 'default :background (car treemacs-window-background-color))
-    (face-remap-add-relative 'fringe  :background (car treemacs-window-background-color))
-    (face-remap-add-relative 'hl-line :background (cdr treemacs-window-background-color)))
+  (face-remap-add-relative 'default 'treemacs-window-background-face)
+  (face-remap-add-relative 'fringe  'treemacs-window-background-face)
+  (face-remap-add-relative 'hl-line 'treemacs-hl-line-face)
 
   (when treemacs-text-scale
     (text-scale-increase treemacs-text-scale))
@@ -319,11 +389,8 @@ Will simply return `treemacs--eldoc-msg'."
 
   (treemacs--build-indentation-cache 6)
   (treemacs--select-icon-set)
-  (treemacs--setup-icon-highlight)
-  (treemacs--setup-icon-background-colors)
   (treemacs--setup-mode-line)
-  (treemacs--reset-dom)
-  (treemacs--reset-project-positions))
+  (treemacs--reset-dom))
 
 (defun treemacs--mode-check-advice (mode-activation &rest args)
   "Verify that `treemacs-mode' is called in the right place.
