@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -43,6 +43,17 @@
 (defvar treemacs--mouse-project-list-functions
   '(("Add Project.el project" . treemacs--builtin-project-mouse-selection-menu)))
 
+(defun treemacs--mouse-drag-advice (fn &rest args)
+  "Advice to wrap `adjust-window-trailing-edge' as FN and its ARGS.
+Ensure that treemacs' window width can be changed with the mouse, even if it is
+locked."
+  (with-selected-window (or (treemacs-get-local-window) (selected-window))
+    (let ((treemacs--width-is-locked)
+          (window-size-fixed))
+      (apply fn args))))
+
+(advice-add #'adjust-window-trailing-edge :around #'treemacs--mouse-drag-advice)
+
 (defun treemacs--builtin-project-mouse-selection-menu ()
   "Build a mouse selection menu for project.el projects."
   (pcase (if (fboundp 'project-known-project-roots)
@@ -84,12 +95,13 @@ Must be bound to a mouse click, or EVENT will not be supplied."
 ;;;###autoload
 (defun treemacs-doubleclick-action (event)
   "Run the appropriate double-click action for the current node.
-In the default configuration this means to do the same as `treemacs-RET-action'.
+In the default configuration this means to expand/collapse directories and open
+files and tags in the most recently used window.
 
 This function's exact configuration is stored in
 `treemacs-doubleclick-actions-config'.
 
-Must be bound to a mouse click, or EVENT will not be supplied."
+Must be bound to a mouse double click to properly handle a click EVENT."
   (interactive "e")
   (when (eq 'double-mouse-1 (elt event 0))
     (goto-char (posn-point (cadr event)))
@@ -136,13 +148,52 @@ Clicking on icons will expand a file's tags, just like
 Must be bound to a mouse click, or EVENT will not be supplied."
   (interactive "e")
   (when (eq 'drag-mouse-1 (elt event 0))
-    (-when-let (treemacs-buffer (treemacs-get-local-buffer))
-      (let* ((node (with-current-buffer treemacs-buffer (treemacs-node-at-point)))
-             (path (-some-> node (treemacs-button-get :path))))
-        (treemacs-with-path path
-          :file-action (progn (select-window (elt (elt event 2) 0))
-                              (find-file path))
-          :no-match-action (ignore))))))
+    (let* ((info1 (elt (cdr event) 0))
+           (info2 (elt (cdr event) 1))
+           (source-window (elt info1 0))
+           (target-window (elt info2 0))
+           (source-pos (elt info1 1))
+           (target-pos (elt info2 1))
+           (treemacs-buffer (treemacs-get-local-buffer)))
+      (if (eq source-window target-window)
+          (treemacs--drag-move-files source-pos target-pos)
+          (let* ((node (with-current-buffer treemacs-buffer (treemacs-node-at-point)))
+                 (path (-some-> node (treemacs-button-get :path))))
+            (treemacs-with-path path
+              :file-action (progn (select-window target-window)
+                                  (find-file path))
+              :no-match-action (ignore)))))))
+
+(defun treemacs--drag-move-files (source-pos target-pos)
+  "Move files with a mouse-drag action.
+SOURCE-POS: Start position of the mouse drag.
+TARGET-POS: End position of the mouse drag."
+  (let* ((source-btn (treemacs--button-in-line source-pos))
+         (target-btn (treemacs--button-in-line target-pos))
+         (source-key (-some-> source-btn (treemacs-button-get :key)))
+         (target-key (-some-> target-btn (treemacs-button-get :key)))
+         (target-dir (and target-key
+                          (if (file-directory-p target-key)
+                              target-key
+                            (treemacs--parent-dir target-key))))
+         (target-file (and source-key target-key
+                           (treemacs-join-path target-dir (treemacs--filename source-key)))))
+    (when (and treemacs-move-files-by-mouse-dragging
+               source-key target-key
+               (not (string= source-key target-key))
+               (not (treemacs-is-path source-key :directly-in target-dir)))
+      (treemacs-do-delete-single-node source-key)
+      (treemacs--without-filewatch
+       (rename-file source-key target-file))
+      (run-hook-with-args 'treemacs-copy-file-functions source-key target-dir)
+      (treemacs-do-insert-single-node target-file target-dir)
+      (treemacs-update-single-file-git-state source-key)
+      (treemacs-update-single-file-git-state target-file)
+      (treemacs--on-file-deletion source-key)
+      (treemacs-goto-file-node target-file)
+      (treemacs-pulse-on-success "Moved %s to %s"
+        (propertize (treemacs--filename target-file) 'face 'font-lock-string-face)
+        (propertize target-dir 'face 'font-lock-string-face)))))
 
 ;;;###autoload
 (defun treemacs-define-doubleclick-action (state action)
@@ -172,7 +223,7 @@ and ignore any prefix argument."
      :no-match-explanation "")))
 
 (defun treemacs--imenu-tag-noselect (file tag-path)
-  "Return a list of the source buffer for FILE and the position of the tag from TAG-PATH."
+  "Return a list of the source buffer for FILE and the tag's from TAG-PATH."
   (let ((tag (-last-item tag-path))
         (path (-butlast tag-path)))
     (condition-case e
@@ -227,7 +278,7 @@ and ignore any prefix argument."
                (list (xref-item-buffer xref) (xref-item-position xref)))))
           ('issue-warning
            (treemacs-log-failure "Tag '%s' is located in a buffer that does not exist."
-                          (propertize (treemacs-with-button-buffer btn (treemacs--get-label-of btn)) 'face 'treemacs-tags-face)))
+             (propertize (treemacs-with-button-buffer btn (treemacs--get-label-of btn)) 'face 'treemacs-tags-face)))
           (_ (error "[Treemacs] '%s' is an invalid value for treemacs-goto-tag-strategy" treemacs-goto-tag-strategy)))))))
 
 ;;;###autoload
@@ -250,7 +301,10 @@ and ignore any prefix argument."
             (menu
              (easy-menu-create-menu
               nil
-              `(("New"
+              `(["Paste here"
+                 treemacs-paste-dir-at-point-to-minibuffer
+                 :visible ,(string-match-p "\\(\\(Move\\)\\|\\(Copy\\)\\) to: " (or (minibuffer-prompt) ""))]
+                ("New"
                  ["New File"      treemacs-create-file]
                  ["New Directory" treemacs-create-dir])
                 ["Open"   treemacs-visit-node-no-split :visible ,(check node)]
@@ -265,15 +319,16 @@ and ignore any prefix argument."
                 ["Open Tags"  treemacs-toggle-node :visible ,(check (memq state '(file-node-closed tag-node-closed)))]
                 ["Close Tags" treemacs-toggle-node :visible ,(check (memq state '(file-node-open tag-node-open)))]
 
-                ["--" #'ignore                         :visible ,(check node)]
-                ["Rename"           treemacs-rename    :visible ,(check node)]
-                ["Delete"           treemacs-delete    :visible ,(check node)]
-                ["Move"             treemacs-move-file :visible ,(check node)]
+                ["--" #'ignore                           :visible ,(check node)]
+                ["Rename"           treemacs-rename-file :visible ,(check node)]
+                ["Delete"           treemacs-delete-file :visible ,(check node)]
+                ["Move"             treemacs-move-file   :visible ,(check node)]
                 ("Copy"
                  ["Copy File"          treemacs-copy-file                   :visible ,(check node)]
                  ["Copy Absolute Path" treemacs-copy-absolute-path-at-point :visible ,(check node)]
                  ["Copy Relative Path" treemacs-copy-relative-path-at-point :visible ,(check node)]
-                 ["Copy Project Path"  treemacs-copy-project-path-at-point  :visible ,(check node)])
+                 ["Copy Project Path"  treemacs-copy-project-path-at-point  :visible ,(check node)]
+                 ["Copy Filename"      treemacs-copy-filename-at-point      :visible ,(check node)])
 
                 ["--" #'ignore t]
                 ("Projects"
@@ -310,7 +365,7 @@ and ignore any prefix argument."
             (cmd (lookup-key menu (apply 'vector choice))))
        ;; In the terminal clicking on a nested menu item does not expand it, but actually
        ;; selects it as the chosen use option.  So as a workaround we need to manually go
-       ;; thtough the menus until we land on an executable command.
+       ;; through the menus until we land on an executable command.
        (while (and (not (commandp cmd))
                    (not (eq cmd menu)))
          (setf menu choice
