@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -33,12 +33,16 @@
 (require 'treemacs-workspaces)
 (require 'treemacs-visuals)
 (require 'treemacs-logging)
+(require 'treemacs-annotations)
 
 (eval-when-compile
   (require 'cl-lib)
   (require 'treemacs-macros)
   (require 'inline))
 
+(treemacs-import-functions-from "treemacs"
+  treemacs-select-window)
+
 (treemacs-import-functions-from "treemacs-filewatch-mode"
   treemacs--start-watching
   treemacs--stop-watching)
@@ -47,20 +51,16 @@
   treemacs--get-indentation)
 
 (treemacs-import-functions-from "treemacs-interface"
+  treemacs-add-project-to-workspace
   treemacs-TAB-action)
 
-(treemacs-import-functions-from "treemacs-extensions"
-  treemacs--apply-root-top-extensions
-  treemacs--apply-root-bottom-extensions
-  treemacs--apply-project-top-extensions
-  treemacs--apply-project-bottom-extensions
-  treemacs--apply-directory-top-extensions
-  treemacs--apply-directory-bottom-extensions)
-
 (treemacs-import-functions-from "treemacs-tags"
   treemacs--expand-file-node
   treemacs--expand-tag-node)
 
+;; Ensure mouse cursor turns into a hand over treemacs' buttons
+(put 'treemacs-button 'mouse-face 'highlight)
+
 (defvar-local treemacs--projects-end nil
   "Marker pointing to position at the end of the last project.
 
@@ -88,11 +88,24 @@ is a marker pointing to POS."
   (inline-letevals (pos)
     (inline-quote (copy-marker ,pos t))))
 
+(define-inline treemacs--button-in-line (pos)
+  "Return the button in the line at POS in the current buffer, or nil.
+If the button at POS is a text property button, the return value
+is a marker pointing to POS."
+  (inline-letevals (pos)
+    (inline-quote
+     (save-excursion
+       (goto-char ,pos)
+       (copy-marker
+        (next-single-property-change
+         (line-beginning-position) 'button nil (line-end-position))
+        t)))))
+
 (define-inline treemacs--current-screen-line ()
   "Get the current screen line in the selected window."
   (declare (side-effect-free t))
   (inline-quote
-   (max 1 (count-screen-lines (window-start) (point-at-eol)))))
+   (max 1 (count-screen-lines (window-start) (line-end-position)))))
 
 (define-inline treemacs--lines-in-window ()
   "Determine the number of lines visible in the current (treemacs) window.
@@ -115,6 +128,18 @@ the height of treemacs' icons must be taken into account."
   (inline-letevals (f1 f2)
     (inline-quote (string-lessp ,f2 ,f1))))
 
+(define-inline treemacs--sort-alphabetic-numeric-asc (f1 f2)
+  "Sort F1 and F2 alphabetically and numerically ascending."
+  (declare (pure t) (side-effect-free t))
+  (inline-letevals (f1 f2)
+    (inline-quote (string-version-lessp ,f1 ,f2))))
+
+(define-inline treemacs--sort-alphabetic-numeric-desc (f1 f2)
+  "Sort F1 and F2 alphabetically and numerically descending."
+  (declare (pure t) (side-effect-free t))
+  (inline-letevals (f1 f2)
+    (inline-quote (string-version-lessp ,f2 ,f1))))
+
 (define-inline treemacs--sort-alphabetic-case-insensitive-asc (f1 f2)
   "Sort F1 and F2 case insensitive alphabetically ascending."
   (declare (pure t) (side-effect-free t))
@@ -127,6 +152,18 @@ the height of treemacs' icons must be taken into account."
   (inline-letevals (f1 f2)
     (inline-quote (string-lessp (downcase ,f2) (downcase ,f1)))))
 
+(define-inline treemacs--sort-alphabetic-numeric-case-insensitive-asc (f1 f2)
+  "Sort F1 and F2 case insensitive alphabetically and numerically ascending."
+  (declare (pure t) (side-effect-free t))
+  (inline-letevals (f1 f2)
+    (inline-quote (string-version-lessp (downcase ,f1) (downcase ,f2)))))
+
+(define-inline treemacs--sort-alphabetic-numeric-case-insensitive-desc (f1 f2)
+  "Sort F1 and F2 case insensitive alphabetically and numerically descending."
+  (declare (pure t) (side-effect-free t))
+  (inline-letevals (f1 f2)
+    (inline-quote (string-version-lessp (downcase ,f2) (downcase ,f1)))))
+
 (define-inline treemacs--sort-size-asc (f1 f2)
   "Sort F1 and F2 by size ascending."
   (declare (side-effect-free t))
@@ -166,8 +203,12 @@ the height of treemacs' icons must be taken into account."
    (pcase treemacs-sorting
      ('alphabetic-asc #'treemacs--sort-alphabetic-asc)
      ('alphabetic-desc #'treemacs--sort-alphabetic-desc)
+     ('alphabetic-numeric-asc #'treemacs--sort-alphabetic-numeric-asc)
+     ('alphabetic-numeric-desc #'treemacs--sort-alphabetic-numeric-desc)
      ('alphabetic-case-insensitive-asc  #'treemacs--sort-alphabetic-case-insensitive-asc)
      ('alphabetic-case-insensitive-desc #'treemacs--sort-alphabetic-case-insensitive-desc)
+     ('alphabetic-numeric-case-insensitive-asc  #'treemacs--sort-alphabetic-numeric-case-insensitive-asc)
+     ('alphabetic-numeric-case-insensitive-desc #'treemacs--sort-alphabetic-numeric-case-insensitive-desc)
      ('size-asc #'treemacs--sort-size-asc)
      ('size-desc #'treemacs--sort-size-desc)
      ('mod-time-asc #'treemacs--sort-mod-time-asc)
@@ -192,23 +233,24 @@ the height of treemacs' icons must be taken into account."
 PREFIX is a string inserted as indentation.
 PARENT is the (optional) button under which this one is inserted.
 DEPTH indicates how deep in the filetree the current button is."
-  ;; for directories the icon is included in the prefix since it's always known
   (inline-letevals (path prefix parent depth)
     (inline-quote
-     (list
-      ,prefix
-      (propertize (->> ,path file-name-nondirectory (funcall treemacs-directory-name-transformer))
-                  'button '(t)
-                  'category 'default-button
-                  'help-echo nil
-                  'keymap nil
-                  :default-face 'treemacs-directory-face
-                  :state 'dir-node-closed
-                  :path ,path
-                  :key ,path
-                  :symlink (file-symlink-p ,path)
-                  :parent ,parent
-                  :depth ,depth)))))
+     (let ((dir-name (file-name-nondirectory ,path)))
+       (list
+        ,prefix
+        (treemacs-icon-for-dir dir-name 'closed)
+        (propertize (->> dir-name (funcall treemacs-directory-name-transformer))
+                    'button '(t)
+                    'category 'treemacs-button
+                    'help-echo nil
+                    'keymap nil
+                    :default-face 'treemacs-directory-face
+                    :state 'dir-node-closed
+                    :path ,path
+                    :key ,path
+                    :symlink (file-symlink-p ,path)
+                    :parent ,parent
+                    :depth ,depth))))))
 
 (define-inline treemacs--create-file-button-strings (path prefix parent depth)
   "Return the text to insert for a file button for PATH.
@@ -222,7 +264,7 @@ DEPTH indicates how deep in the filetree the current button is."
       (treemacs-icon-for-file ,path)
       (propertize (->> ,path file-name-nondirectory (funcall treemacs-file-name-transformer))
                   'button '(t)
-                  'category 'default-button
+                  'category 'treemacs-button
                   'help-echo nil
                   'keymap nil
                   :default-face 'treemacs-git-unmodified-face
@@ -236,9 +278,9 @@ DEPTH indicates how deep in the filetree the current button is."
 (cl-defmacro treemacs--button-open (&key button new-state new-icon open-action post-open-action immediate-insert)
   "Building block macro to open a BUTTON.
 Gives the button a NEW-STATE, and, optionally, a NEW-ICON.  Performs OPEN-ACTION
-and, optionally, POST-OPEN-ACTION.  If IMMEDIATE-INSERT is non-nil it will concat
-and apply `insert' on the items returned from OPEN-ACTION.  If it is nil either
-OPEN-ACTION or POST-OPEN-ACTION are expected to take over insertion."
+and, optionally, POST-OPEN-ACTION.  If IMMEDIATE-INSERT is non-nil it will
+concat and apply `insert' on the items returned from OPEN-ACTION.  If it is nil
+either OPEN-ACTION or POST-OPEN-ACTION are expected to take over insertion."
   `(prog1
      (save-excursion
        (let ((p (point))
@@ -248,7 +290,7 @@ OPEN-ACTION or POST-OPEN-ACTION are expected to take over insertion."
           ,@(when new-icon
               `((beginning-of-line)
                 (treemacs--button-symbol-switch ,new-icon)))
-          (goto-char (treemacs-button-end ,button))
+          (goto-char (line-end-position))
           ,@(if immediate-insert
                 `((progn
                     (insert (apply #'concat ,open-action))))
@@ -336,10 +378,18 @@ DIRS: List of Collapse Paths.  Each Collapse Path is a list of
               (-let [beg (point)]
                 (insert label-to-add)
                 (add-text-properties beg (point) props)
-                (unless (memq treemacs--git-mode '(deferred extended))
+                (unless (treemacs--non-simple-git-mode-enabled)
                   (add-text-properties
                    beg (point)
-                   '(face treemacs-directory-collapsed-face)))))))))))
+                   '(face treemacs-directory-collapsed-face)))
+                (-when-let* ((ann (treemacs-get-annotation new-path))
+                             (git-cache
+                              (->> original-path
+                                   (treemacs--parent-dir)
+                                   (ht-get treemacs--git-cache))))
+                  (treemacs-button-put
+                   b 'face
+                   (treemacs-annotation->face-value ann)))))))))))
 
 (defmacro treemacs--inplace-map-when-unrolled (items interval &rest mapper)
   "Unrolled in-place mapping operation.
@@ -353,17 +403,19 @@ Maps ITEMS at given index INTERVAL using MAPPER function."
                    (_ (error "Interval %s is not handled yet" interval)))))
     `(let ((,l ,items))
        (while ,l
-         (setq ,l (,tail-op ,l))
-         (let ((it (pop ,l)))
-           ,@mapper)))))
+          (setq ,l (,tail-op ,l))
+          (let ((it (car ,l)))
+            (setf (car ,l) ,@mapper)
+            (pop ,l)))
+       ,items)))
 
-(define-inline treemacs--create-branch (root depth git-future collapse-process &optional parent)
+(define-inline treemacs--create-branch (root depth git-future flatten-future &optional parent)
   "Create a new treemacs branch under ROOT.
 The branch is indented at DEPTH and uses the eventual outputs of
-GIT-FUTURE to decide on file buttons' faces and COLLAPSE-PROCESS to determine
+GIT-FUTURE to decide on file buttons' faces and FLATTEN-FUTURE to determine
 which directories should be displayed as one.  The buttons' parent property is
 set to PARENT."
-  (inline-letevals (root depth git-future collapse-process parent)
+  (inline-letevals (root depth git-future flatten-future parent)
     (inline-quote
      (save-excursion
        (let* ((dirs-and-files (treemacs--get-dir-content ,root))
@@ -378,10 +430,9 @@ set to PARENT."
          (setq dir-strings
                (treemacs--create-buttons
                 :nodes dirs
-                :extra-vars ((dir-prefix (concat prefix treemacs-icon-dir-closed)))
                 :depth ,depth
                 :node-name node
-                :node-action (treemacs--create-dir-button-strings node dir-prefix ,parent ,depth)))
+                :node-action (treemacs--create-dir-button-strings node prefix ,parent ,depth)))
          (setq file-strings
                (treemacs--create-buttons
                 :nodes files
@@ -408,10 +459,14 @@ set to PARENT."
             (setf git-info (treemacs--get-or-parse-git-result ,git-future))
             (ht-set! treemacs--git-cache ,root git-info))
            ('deferred
-             (setq git-info (or (ht-get treemacs--git-cache ,root) (ht)))
-             (run-with-timer 0.5 nil #'treemacs--apply-deferred-git-state ,parent ,git-future (current-buffer)))
+             (setf git-info (or (ht-get treemacs--git-cache ,root) treemacs--empty-table)))
            (_
-            (setq git-info (ht))))
+            (setf git-info treemacs--empty-table)))
+
+         (run-with-timer
+          0.5 nil
+          #'treemacs--apply-annotations-deferred
+          ,parent ,root (current-buffer) ,git-future)
 
          (if treemacs-pre-file-insert-predicates
              (progn
@@ -430,13 +485,14 @@ set to PARENT."
                (-let [result nil]
                  (while dir-strings
                    (let* ((prefix (car dir-strings))
-                          (dirname (cadr dir-strings))
+                          (icon (cadr dir-strings))
+                          (dirname (caddr dir-strings))
                           (dirpath (concat ,root "/" dirname)))
                      (unless (--any? (funcall it dirpath git-info) treemacs-pre-file-insert-predicates)
-                       (setq result (cons dirname (cons prefix result)))
+                       (setq result (cons dirname (cons icon (cons prefix result))))
                        (push (treemacs-dom-node->create! :parent parent-node :key dirpath)
                              dir-dom-nodes)))
-                   (setq dir-strings (cddr dir-strings)))
+                   (setq dir-strings (cdddr dir-strings)))
                  (setq dir-strings (nreverse result))))
            (setf
             file-dom-nodes
@@ -451,29 +507,55 @@ set to PARENT."
          (dolist (it (treemacs-dom-node->children parent-node))
            (treemacs-dom-node->insert-into-dom! it))
 
-         (treemacs--inplace-map-when-unrolled dir-strings 2
-           (put-text-property
-            0
-            (length it)
-            'face
-            (treemacs--get-node-face (concat ,root "/" it) git-info 'treemacs-directory-face)
-            it))
+         (setf dir-strings
+               (treemacs--inplace-map-when-unrolled dir-strings 3
+                 (-if-let* ((ann (treemacs-get-annotation (concat ,root "/" it)))
+                            (face (treemacs-annotation->face-value ann)))
+                     (progn
+                       (put-text-property
+                        0
+                        (length it)
+                        'face
+                        face
+                        it)
+                       (concat it (treemacs-annotation->suffix-value ann)))
+                   (put-text-property
+                    0
+                    (length it)
+                    'face
+                    'treemacs-directory-face
+                    it)
+                   it)))
          (insert (apply #'concat dir-strings))
 
          (end-of-line)
-         (treemacs--inplace-map-when-unrolled file-strings 3
-           (put-text-property
-            0
-            (length it)
-            'face
-            (treemacs--get-node-face (concat ,root "/" it) git-info 'treemacs-git-unmodified-face)
-            it))
+         (setf file-strings
+               (treemacs--inplace-map-when-unrolled file-strings 3
+                 (-if-let* ((ann (treemacs-get-annotation (concat ,root "/" it)))
+                            (face (treemacs-annotation->face-value ann)))
+                     (progn
+                       (put-text-property
+                        0
+                        (length it)
+                        'face
+                        face
+                        it)
+                       (concat it (treemacs-annotation->suffix-value ann)))
+                   (put-text-property
+                    0
+                    (length it)
+                    'face
+                    'treemacs-git-unmodified-face
+                    it)
+                   it)))
          (insert (apply #'concat file-strings))
 
          (save-excursion
-           (treemacs--flatten-dirs (treemacs--parse-collapsed-dirs ,collapse-process))
-           (treemacs--reentry ,root ,git-future))
-         (point-at-eol))))))
+           (treemacs--flatten-dirs
+            (treemacs--parse-flattened-dirs ,root ,flatten-future))
+           (treemacs--reentry ,root ,git-future ,flatten-future))
+         (with-no-warnings
+           (line-end-position)))))))
 
 (cl-defmacro treemacs--button-close (&key button new-icon new-state post-close-action)
   "Close node given by BUTTON, use NEW-ICON and BUTTON's state to NEW-STATE.
@@ -483,7 +565,7 @@ Run POST-CLOSE-ACTION after everything else is done."
       ,@(when new-icon
           `((treemacs--button-symbol-switch ,new-icon)))
       (treemacs-button-put ,button :state ,new-state)
-      (-let [next (next-button (point-at-eol))]
+      (-let [next (next-button (button-end ,button))]
         (if (or (null next)
                 (/= (1+ (treemacs-button-get ,button :depth))
                     (treemacs-button-get (copy-marker next t) :depth)))
@@ -493,7 +575,7 @@ Run POST-CLOSE-ACTION after everything else is done."
           ;; current button, making the treemacs--projects-end marker track
           ;; properly when collapsing the last project or a last directory of the
           ;; last project.
-          (let* ((pos-start (treemacs-button-end ,button))
+          (let* ((pos-start (line-end-position))
                  (next (treemacs--next-non-child-button ,button))
                  (pos-end (if next
                               (-> next (treemacs-button-start) (previous-button) (treemacs-button-end))
@@ -517,7 +599,8 @@ RECURSIVE: Bool"
       (let* ((path (treemacs-button-get btn :path))
              (git-path (if (treemacs-button-get btn :symlink) (file-truename path) path))
              (git-future (treemacs--git-status-process git-path project))
-             (collapse-future (treemacs--collapsed-dirs-process path project)))
+             (flatten-future (treemacs--flattened-dirs-process path project))
+             (recursive (treemacs--prefix-arg-to-recurse-depth recursive)) )
         (treemacs--maybe-recenter treemacs-recenter-after-project-expand
           (treemacs--button-open
            :immediate-insert nil
@@ -529,19 +612,28 @@ RECURSIVE: Bool"
              ;; TODO(2019/10/14): go back to post open
              ;; expand first because it creates a dom node entry
              (treemacs-on-expand path btn)
-             (treemacs--apply-project-top-extensions btn project)
-             (goto-char (treemacs--create-branch path (1+ (treemacs-button-get btn :depth)) git-future collapse-future btn))
-             (treemacs--apply-project-bottom-extensions btn project)
+             (when (fboundp 'treemacs--apply-project-top-extensions)
+               (treemacs--apply-project-top-extensions btn project))
+             (when (fboundp 'treemacs--apply-project-bottom-extensions)
+               (save-excursion
+                 (treemacs--apply-project-bottom-extensions btn project)))
+             (treemacs--create-branch
+              path
+              (1+ (treemacs-button-get btn :depth))
+              git-future
+              flatten-future
+              btn)
              (treemacs--start-watching path)
              ;; Performing FS ops on a disconnected Tramp project
              ;; might have changed the state to connected.
              (treemacs-with-writable-buffer
               (treemacs-project->refresh-path-status! project))
-             (when (and recursive (treemacs-project->is-readable? project))
+             (when (and (> recursive 0) (treemacs-project->is-readable? project))
+               (cl-decf recursive)
                (--each (treemacs-collect-child-nodes btn)
                  (when (eq 'dir-node-closed (treemacs-button-get it :state))
                    (goto-char (treemacs-button-start it))
-                   (treemacs--expand-dir-node it :git-future git-future :recursive t)))))))))))
+                   (treemacs--expand-dir-node it :git-future git-future :recursive recursive)))))))))))
 
 (defun treemacs--collapse-root-node (btn &optional recursive)
   "Collapse the given root BTN.
@@ -555,51 +647,68 @@ Remove all open entries below BTN when RECURSIVE is non-nil."
      (treemacs--stop-watching path)
      (treemacs-on-collapse path recursive))))
 
-(cl-defun treemacs--expand-dir-node (btn &key git-future recursive)
+(cl-defun treemacs--expand-dir-node
+    (btn
+     &key
+     git-future
+     flatten-future
+     recursive)
   "Open the node given by BTN.
 
 BTN: Button
 GIT-FUTURE: Pfuture|HashMap
+FLATTEN-FUTURE: Pfuture|HashMap
 RECURSIVE: Bool"
   (-let [path (treemacs-button-get btn :path)]
     (if (not (file-readable-p path))
-        (treemacs-pulse-on-failure
-            "Directory %s is not readable." (propertize path 'face 'font-lock-string-face))
+        (treemacs-pulse-on-failure "Directory %s is not readable."
+          (propertize path 'face 'font-lock-string-face))
       (let* ((project (treemacs-project-of-node btn))
              (git-future (if (treemacs-button-get btn :symlink)
                              (treemacs--git-status-process (file-truename path) project)
                            (or git-future (treemacs--git-status-process path project))))
-             (collapse-future (treemacs--collapsed-dirs-process path project)))
+             (flatten-future (or flatten-future
+                                 (treemacs--flattened-dirs-process path project)))
+             (recursive (treemacs--prefix-arg-to-recurse-depth recursive))
+             (base-dir-name (treemacs--filename (treemacs-button-get btn :key))))
         (treemacs--button-open
          :immediate-insert nil
          :button btn
          :new-state 'dir-node-open
-         :new-icon treemacs-icon-dir-open
+         :new-icon (treemacs-icon-for-dir base-dir-name 'open)
          :open-action
          (progn
            ;; do on-expand first so buttons that need collapsing can quickly find their parent
            (treemacs-on-expand path btn)
-           (treemacs--apply-directory-top-extensions btn path)
-           (goto-char (treemacs--create-branch path (1+ (treemacs-button-get btn :depth)) git-future collapse-future btn))
-           (treemacs--apply-directory-bottom-extensions btn path)
+           (when (fboundp 'treemacs--apply-directory-top-extensions)
+             (treemacs--apply-directory-top-extensions btn path))
+           (goto-char
+            (treemacs--create-branch
+             path (1+ (treemacs-button-get btn :depth))
+             git-future flatten-future btn))
+           (when (fboundp 'treemacs--apply-directory-bottom-extensions)
+             (treemacs--apply-directory-bottom-extensions btn path))
            (treemacs--start-watching path)
-           (when recursive
+           (when (> recursive 0)
+             (cl-decf recursive)
              (--each (treemacs-collect-child-nodes btn)
                (when (eq 'dir-node-closed (treemacs-button-get it :state))
                  (goto-char (treemacs-button-start it))
-                 (treemacs--expand-dir-node it :git-future git-future :recursive t))))))))))
+                 (treemacs--expand-dir-node it :git-future git-future :recursive recursive))))))))))
 
 (defun treemacs--collapse-dir-node (btn &optional recursive)
   "Close node given by BTN.
 Remove all open dir and tag entries under BTN when RECURSIVE."
-  (treemacs--button-close
-   :button btn
-   :new-state 'dir-node-closed
-   :new-icon treemacs-icon-dir-closed
-   :post-close-action
-   (-let [path (treemacs-button-get btn :path)]
-     (treemacs--stop-watching path)
-     (treemacs-on-collapse path recursive))))
+  (let ((path (treemacs-button-get btn :path))
+        (base-dir-name (treemacs--filename (treemacs-button-get btn :key))))
+    (treemacs--button-close
+     :button btn
+     :new-state 'dir-node-closed
+     :new-icon (treemacs-icon-for-dir base-dir-name 'closed)
+     :post-close-action
+     (progn
+       (treemacs--stop-watching path)
+       (treemacs-on-collapse path recursive)))))
 
 (defun treemacs--root-face (project)
   "Get the face to be used for PROJECT."
@@ -619,13 +728,14 @@ PROJECT: Project Struct"
          (path (treemacs-project->path project))
          (dom-node (treemacs-dom-node->create! :key path :position pos)))
     (treemacs-dom-node->insert-into-dom! dom-node)
-    (treemacs--set-project-position project pos)
     (insert
      (propertize (treemacs-project->name project)
                  'button '(t)
-                 'category 'default-button
+                 'category 'treemacs-button
                  'face (treemacs--root-face project)
                  :project project
+                 :default-face 'treemacs-root-face
+                 :key path
                  :symlink (when (treemacs-project->is-readable? project)
                             (file-symlink-p path))
                  :state 'root-node-closed
@@ -639,7 +749,8 @@ PROJECT: Project Struct"
      (setq treemacs--projects-end (make-marker)))
    (let* ((projects (-reject #'treemacs-project->is-disabled? projects))
           (current-workspace (treemacs-current-workspace))
-          (has-previous (treemacs--apply-root-top-extensions current-workspace)))
+          (has-previous (when (fboundp 'treemacs--apply-root-top-extensions)
+                          (treemacs--apply-root-top-extensions current-workspace))))
 
      (--each projects
        (when has-previous (treemacs--insert-root-separator))
@@ -649,7 +760,8 @@ PROJECT: Project Struct"
      ;; Set the end marker after inserting the extensions. Otherwise, the
      ;; extensions would move the marker.
      (let ((projects-end-point (point)))
-       (treemacs--apply-root-bottom-extensions current-workspace has-previous)
+       (when (fboundp 'treemacs--apply-root-bottom-extensions)
+         (treemacs--apply-root-bottom-extensions current-workspace has-previous))
        ;; If the marker lies at the start of the buffer, expanding extensions would
        ;; move the marker. Make sure that the marker does not move when doing so.
        (set-marker-insertion-type treemacs--projects-end has-previous)
@@ -937,7 +1049,7 @@ FLATTEN-INFO [Int File Path...]"
 
          ;; Insert new label
          (goto-char parent-btn)
-         (delete-region (point) (point-at-eol))
+         (delete-region (point) (line-end-position))
          (insert (apply #'propertize new-button-label properties))
 
          ;; Fixing marker probably necessary since it's also in the dom
@@ -970,13 +1082,12 @@ DEPTH: Int"
         (let* ((strs)
                (face))
           (if (file-directory-p ,path)
-              (setf strs (treemacs--create-dir-button-strings
-                          ,path
-                          (concat prefix treemacs-icon-dir-closed)
-                          ,parent
-                          ,depth)
-                    face 'treemacs-directory-face)
-            (setf strs (treemacs--create-file-button-strings ,path prefix ,parent ,depth)
+              (setf
+               strs (treemacs--create-dir-button-strings
+                     ,path prefix ,parent ,depth)
+               face 'treemacs-directory-face)
+            (setf strs (treemacs--create-file-button-strings
+                        ,path prefix ,parent ,depth)
                   face 'treemacs-file-face))
           (-let [last (-last-item strs)]
             (put-text-property 0 (length last) 'face face last))
@@ -1039,8 +1150,7 @@ parents' git status can be updated."
                (treemacs-do-delete-single-node path project))
               ('changed
                (treemacs-do-update-node path)
-               (when (memq treemacs--git-mode '(extended deferred))
-                 (treemacs-update-single-file-git-state path)))
+               (treemacs-update-single-file-git-state path))
               ('created
                (treemacs-do-insert-single-node path (treemacs-dom-node->key node)))
               ('force-refresh
@@ -1086,9 +1196,9 @@ PATH: Node Path"
                                 (treemacs--filename path))))
          t)))))
 
-(defun treemacs--reentry (path &optional git-info)
+(defun treemacs--reentry (path &optional git-future flatten-future)
   "Reopen dirs below PATH.
-GIT-INFO is passed through from the previous branch build.
+GIT-FUTURE and FLATTEN-FUTURE are passed through from the previous branch build.
 
 PATH: Node Path
 GIT-INFO: Pfuture | Map<String, String>"
@@ -1108,17 +1218,42 @@ GIT-INFO: Pfuture | Map<String, String>"
           ;; so the process can continue
           (setf (treemacs-dom-node->reentry-nodes actual-dom-node)
                 (treemacs-dom-node->reentry-nodes to-reopen-dom-node))
-          (treemacs--reopen-node (treemacs-goto-node reopen-path) git-info))))))
+          (treemacs--reopen-node
+           (treemacs-goto-node reopen-path)
+           git-future
+           flatten-future))))))
 
-(defun treemacs--reopen-node (btn &optional git-info)
+(defun treemacs--reopen-node (btn &optional git-future flatten-future)
   "Reopen file BTN.
-GIT-INFO is passed through from the previous branch build."
+GIT-FUTURE and FLATTEN-FUTURE are passed through from the previous branch build."
   (pcase (treemacs-button-get btn :state)
-    ('dir-node-closed  (treemacs--expand-dir-node btn :git-future git-info))
+    ('dir-node-closed  (treemacs--expand-dir-node
+                        btn
+                        :git-future git-future
+                        :flatten-future flatten-future))
     ('file-node-closed (treemacs--expand-file-node btn))
     ('tag-node-closed  (treemacs--expand-tag-node btn))
     ('root-node-closed (treemacs--expand-root-node btn))
-    (other             (funcall (alist-get other treemacs-TAB-actions-config) btn))))
+    (other             (funcall (alist-get other treemacs-TAB-actions-config)))))
+
+(defun treemacs--show-single-project (path name)
+  "Show only a project for the given PATH and NAME in the current workspace."
+  (let* ((ws (treemacs-current-workspace)))
+    (setf  (treemacs-workspace->projects ws)
+           (list (treemacs-project->create!
+                  :name name
+                  :path path
+                  :path-status (treemacs--get-path-status path))))
+    (--when-let (treemacs-get-local-buffer)
+      (with-current-buffer it
+        (treemacs--consolidate-projects)))
+    (-let [treemacs-select-when-already-in-treemacs 'stay]
+      (treemacs-select-window))
+    (goto-char (point-min))
+    (-if-let (btn (treemacs-current-button))
+      (unless (treemacs-is-node-expanded? btn)
+        (treemacs--expand-root-node btn)))
+    (treemacs--evade-image)))
 
 (provide 'treemacs-rendering)
 
