@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -45,9 +45,6 @@
 (treemacs-import-functions-from "treemacs-filewatch-mode"
   treemacs--stop-filewatch-for-current-buffer)
 
-(treemacs-import-functions-from "treemacs-visuals"
-  treemacs--tear-down-icon-highlight)
-
 (treemacs-import-functions-from "treemacs-interface"
   treemacs-quit
   treemacs-select-window)
@@ -55,7 +52,6 @@
 (treemacs-import-functions-from "treemacs-workspaces"
   treemacs--find-workspace)
 
-
 (cl-defstruct (treemacs-scope-shelf
                (:conc-name treemacs-scope-shelf->)
                (:constructor treemacs-scope-shelf->create!))
@@ -181,13 +177,13 @@ NEW-SCOPE-TYPE: T: treemacs-scope"
 
 (defun treemacs--on-buffer-kill ()
   "Cleanup to run when a treemacs buffer is killed."
-  ;; stop watch must come first since we need a reference to the killed buffer
-  ;; to remove it from the filewatch list
-  (treemacs--stop-filewatch-for-current-buffer)
-  (treemacs--tear-down-icon-highlight)
-  ;; not present for extension buffers
-  (-when-let (shelf (treemacs-current-scope-shelf))
-    (setf (treemacs-scope-shelf->buffer shelf) nil)))
+  (when (eq t treemacs--in-this-buffer)
+    ;; stop watch must come first since we need a reference to the killed buffer
+    ;; to remove it from the filewatch list
+    (treemacs--stop-filewatch-for-current-buffer)
+    ;; not present for extension buffers
+    (-when-let (shelf (treemacs-current-scope-shelf))
+      (setf (treemacs-scope-shelf->buffer shelf) nil))))
 
 (defun treemacs--on-scope-kill (scope)
   "Kill and remove the buffer assigned to the given SCOPE."
@@ -256,7 +252,7 @@ Returns nil if no treemacs buffer is visible."
 
 (define-inline treemacs-current-visibility ()
   "Return whether the current visibility state of the treemacs buffer.
-Valid states are 'visible, 'exists and 'none."
+Valid states are \\='visible, \\='exists and \\='none."
   (declare (side-effect-free t))
   (inline-quote
    (cond
