@@ -1,6 +1,6 @@
 ;;; treemacs.el --- A tree style file viewer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 (require 'treemacs-core-utils)
 (require 'treemacs-dom)
 (require 'treemacs-scope)
+(require 'treemacs-customization)
 
 (eval-when-compile
   (require 'cl-lib)
@@ -57,7 +58,6 @@
   treemacs--maybe-load-workspaces)
 
 (treemacs-import-functions-from "treemacs-visuals"
-  treemacs--forget-last-highlight
   treemacs-pulse-on-failure)
 
 (treemacs-import-functions-from "treemacs-async"
@@ -75,9 +75,11 @@
                (:conc-name treemacs-workspace->)
                (:constructor treemacs-workspace->create!))
   name
-  projects)
+  projects
+  is-disabled?)
 
 (defvar treemacs--workspaces (list (treemacs-workspace->create! :name "Default")))
+(defvar treemacs--disabled-workspaces (list))
 
 (defvar treemacs--find-user-project-functions
   (list #'treemacs--current-builtin-project-function
@@ -87,8 +89,6 @@
 (defvar-local treemacs--org-err-ov nil
   "The overlay that will display validations when org-editing.")
 
-(defvar-local treemacs--project-positions nil)
-
 (defvar-local treemacs--project-of-buffer nil
   "The project that the current buffer falls under, if any.")
 
@@ -103,12 +103,16 @@ rely on the current buffer and workspace being aligned.")
 To be called whenever a project or workspace changes."
   (inline-quote
   (dolist (buf (buffer-list))
-    (setf (buffer-local-value 'treemacs--project-of-buffer buf) nil))))
+    (with-current-buffer buf
+      (setf treemacs--project-of-buffer nil)))))
 
 (defun treemacs--current-builtin-project-function ()
   "Find the current project.el project."
   (declare (side-effect-free t))
-  (-some-> (project-current) (cdr) (file-truename) (treemacs-canonical-path)))
+  (-when-let (project (project-current))
+    (if (fboundp 'project-root)
+        (-> project (project-root) (file-truename) (treemacs-canonical-path))
+      (-> project (cdr) (file-truename) (treemacs-canonical-path)))))
 
 (defun treemacs--current-directory-project-function ()
   "Find the current working directory."
@@ -120,6 +124,11 @@ To be called whenever a project or workspace changes."
   (declare (side-effect-free t))
   (inline-quote treemacs--workspaces))
 
+(define-inline treemacs-disabled-workspaces ()
+  "Return the list of all workspaces in treemacs that are disabled."
+  (declare (side-effect-free t))
+  (inline-quote treemacs--disabled-workspaces))
+
 (defun treemacs-current-workspace ()
   "Get the current workspace.
 The return value can be overridden by let-binding `treemacs-override-workspace'.
@@ -155,16 +164,22 @@ PATH: String"
   (declare (side-effect-free t))
   (inline-letevals (path)
     (inline-quote
-     (setf (treemacs-current-workspace)
-           (or (--first (treemacs-is-path ,path :in-workspace it)
-                        treemacs--workspaces)
-               (car treemacs--workspaces))))))
+     (let ((ws-for-path (--first (treemacs-is-path ,path :in-workspace it)
+                                 treemacs--workspaces)))
+       (setf (treemacs-current-workspace)
+             (pcase-exhaustive treemacs-find-workspace-method
+               ('find-for-file-or-pick-first
+                (or ws-for-path (car treemacs--workspaces)))
+               ('find-for-file-or-manually-select
+                (or ws-for-path (treemacs--select-workspace-by-name)))
+               ('always-ask
+                (treemacs--select-workspace-by-name))))))))
 
 ;; TODO(2020/11/25): NAME
 (define-inline treemacs--find-project-for-buffer (&optional buffer-file)
   "In the current workspace find the project current buffer's file falls under.
 Optionally supply the BUFFER-FILE in case it is not available by calling the
-function `buffer-file-name' (like in dired).
+function `buffer-file-name' (like in Dired).
 
 FILE: Filepath"
   (inline-letevals (buffer-file)
@@ -213,18 +228,13 @@ FILE: Filepath"
   "Get the position of the next project.
 Will return `point-max' if there is no next project."
   (declare (side-effect-free t))
-  (inline-quote (next-single-char-property-change (point-at-eol) :project)))
+  (inline-quote (next-single-char-property-change (line-end-position) :project)))
 
 (define-inline treemacs--prev-project-pos ()
   "Get the position of the next project.
 Will return `point-min' if there is no next project."
   (declare (side-effect-free t))
-  (inline-quote (previous-single-char-property-change (point-at-bol) :project)))
-
-(define-inline treemacs--reset-project-positions ()
-  "Reset `treemacs--project-positions'."
-  (inline-quote
-   (setq treemacs--project-positions (make-hash-table :test #'equal :size 20))))
+  (inline-quote (previous-single-char-property-change (line-beginning-position) :project)))
 
 (define-inline treemacs-project->key (self)
   "Get the hash table key of SELF.
@@ -238,58 +248,50 @@ SELF may be a project struct or a root key of a top level extension."
          (treemacs-project->path ,self)
        ,self))))
 
-(define-inline treemacs--set-project-position (project position)
-  "Insert PROJECT's POSITION into `treemacs--project-positions'."
-  (inline-letevals (project position)
-    (inline-quote
-     (ht-set! treemacs--project-positions (treemacs-project->key ,project) ,position))))
-
 (define-inline treemacs-project->position (self)
   "Return the position of project SELF in the current buffer."
   (declare (side-effect-free t))
   (inline-letevals (self)
     (inline-quote
-     (ht-get treemacs--project-positions (treemacs-project->key ,self)))))
+     (treemacs-dom-node->position
+      (treemacs-find-in-dom (treemacs-project->path ,self))))))
 
 (define-inline treemacs-project->is-expanded? (self)
   "Return non-nil if project SELF is expanded in the current buffer."
   (declare (side-effect-free t))
   (inline-letevals (self)
     (inline-quote
-     (eq 'root-node-open
-         (-> ,self (treemacs-project->position) (treemacs-button-get :state))))))
+     (memq (-> ,self (treemacs-project->position) (treemacs-button-get :state))
+           treemacs--open-node-states))))
 
-(define-inline treemacs-project->refresh-path-status! (self)
+(defun treemacs-project->refresh-path-status! (self)
   "Refresh the path status of project SELF in the current buffer.
 Does not preserve the current position in the buffer."
-  (inline-letevals (self)
-    (inline-quote
-     (let ((old-path-status (treemacs-project->path-status ,self))
-           (new-path-status (treemacs--get-path-status (treemacs-project->path ,self))))
-       (unless (eq old-path-status new-path-status)
-         (setf (treemacs-project->path-status ,self) new-path-status)
-         ;; When the path transforms from unreadable or disconnected to readable,
-         ;; update the :symlink status on its button.
-         (let ((pos (treemacs-project->position ,self))
-               (path (treemacs-project->path ,self)))
-           (when (treemacs-project->is-readable? ,self)
-             (treemacs-button-put pos :symlink (file-symlink-p path)))
-           (treemacs-button-put pos 'face (treemacs--root-face ,self))))))))
+  (let ((old-path-status (treemacs-project->path-status self))
+        (new-path-status (treemacs--get-path-status (treemacs-project->path self))))
+    (unless (eq old-path-status new-path-status)
+      (setf (treemacs-project->path-status self) new-path-status)
+      ;; When the path transforms from unreadable or disconnected to readable,
+      ;; update the :symlink status on its button.
+      (let ((pos (treemacs-project->position self))
+            (path (treemacs-project->path self)))
+        (when (treemacs-project->is-readable? self)
+          (treemacs-button-put pos :symlink (file-symlink-p path)))
+        (treemacs-button-put pos 'face (treemacs--root-face self))))))
 
-(define-inline treemacs-project->refresh! (self)
+;; TODO(2021/08/17): -> rendering
+(defun treemacs-project->refresh! (self)
   "Refresh project SELF in the current buffer.
 Does not preserve the current position in the buffer."
-  (inline-letevals (self)
-    (inline-quote
-     (progn
-       (treemacs-project->refresh-path-status! ,self)
-       (when (treemacs-project->is-expanded? ,self)
-         (let ((root-btn (treemacs-project->position ,self)))
-           (goto-char root-btn)
-           (treemacs--forget-last-highlight)
-           (treemacs--collapse-root-node root-btn)
-           (unless (treemacs-project->is-unreadable? ,self)
-             (treemacs--expand-root-node root-btn))))))))
+  (treemacs-project->refresh-path-status! self)
+  (when (treemacs-project->is-expanded? self)
+    (let ((root-btn (treemacs-project->position self)))
+      (goto-char root-btn)
+      (funcall (alist-get (treemacs-button-get root-btn :state)
+                          treemacs-TAB-actions-config))
+      (unless (treemacs-project->is-unreadable? self)
+        (funcall (alist-get (treemacs-button-get root-btn :state)
+                            treemacs-TAB-actions-config))))))
 
 (define-inline treemacs-project->is-last? (self)
   "Return t when root node of project SELF is the last in the view."
@@ -399,6 +401,7 @@ Returns either
 * `remote-unreadable' when PATH is a remote unreadable file or directory,
 * `remote-disconnected' when PATH is remote, but the connection is down, or
 * `extension' when PATH is not a string."
+  (declare (side-effect-free t))
   (cond
    ((not (stringp path)) 'extension)
    ((not (file-remote-p path))
@@ -408,14 +411,15 @@ Returns either
    (t 'remote-unreadable)))
 
 (define-inline treemacs-project->is-unreadable? (self)
-  "Return t if the project SELF is definitely unreadable.
+  "Return non-nil if the project SELF is definitely unreadable.
 
 If `path-status' of the project is `remote-disconnected', the return value will
 be nil even though the path might still be unreadable.  Does not verify the
-readability, the cached path-state is used."
+readability, the cached path-state is used.  Extension projects will count as
+readable."
   (declare (side-effect-free t))
   (inline-quote (memq (treemacs-project->path-status ,self)
-                      '(local-unreadable remote-unreadable extension))))
+                      '(local-unreadable remote-unreadable))))
 
 (define-inline treemacs-project->is-readable? (self)
   "Return t if the project SELF is definitely readable for file operations.
@@ -515,6 +519,7 @@ NAME: String"
            (when treemacs-expand-added-projects
              (treemacs--expand-root-node (treemacs-project->position project))))))
        (treemacs--persist)
+       (treemacs--invalidate-buffer-project-cache)
        (when (with-no-warnings treemacs-hide-gitignored-files-mode)
          (treemacs--prefetch-gitignore-cache path))
        (run-hook-with-args 'treemacs-create-project-functions project)
@@ -571,8 +576,7 @@ Return values may be as follows:
      'user-cancel)
    (treemacs-run-in-every-buffer
     (treemacs-with-writable-buffer
-     (let* ((project-path (treemacs-project->path project))
-            (project-pos (goto-char (treemacs-project->position project-path)))
+     (let* ((project-pos (goto-char (treemacs-project->position project)))
             (prev-project-pos (move-marker (make-marker) (treemacs--prev-project-pos)))
             (next-project-pos (move-marker (make-marker) (treemacs--next-project-pos))))
        (when (treemacs-project->is-expanded? project)
@@ -586,7 +590,7 @@ Return values may be as follows:
           ;; the end of the previous button's line. If the `treemacs--projects-end'
           ;; is at the EOL of the  it will move to EOL of the previous button.
           (previous-button
-           (delete-region (treemacs-button-end previous-button) (point-at-eol))
+           (delete-region (treemacs-button-end previous-button) (line-end-position))
            (when next-button (forward-button 1)))
           ;; Previous project does not exist, but a next button exists. Delete from
           ;; BOL to the start of the next buttons line.
@@ -595,16 +599,16 @@ Return values may be as follows:
              ;; The first item after the deletion will be bottom extensions. Project
              ;; end will be at its BOL, making it move upon expand/collapse. Lock the marker.
              (set-marker-insertion-type (treemacs--projects-end) nil))
-           (delete-region (point-at-bol) (progn (goto-char next-button) (forward-line 0) (point))))
+           (delete-region (line-beginning-position) (progn (goto-char next-button) (forward-line 0) (point))))
 
           ;; Neither the previous nor the next button exists. Simply delete the
           ;; current line.
           (t
-           (delete-region (point-at-bol) (point-at-eol)))))
+           (delete-region (line-beginning-position) (line-end-position)))))
        (if (equal (point-min) prev-project-pos)
            (goto-char next-project-pos)
          (goto-char prev-project-pos)))
-     (treemacs--forget-last-highlight)
+     (treemacs--invalidate-buffer-project-cache)
      (--when-let (treemacs-get-local-window)
        (with-selected-window it
          (recenter)))
@@ -644,7 +648,10 @@ Return values may be as follows:
        (let* ((workspaces (->> treemacs--workspaces
                                (--reject (eq it (treemacs-current-workspace)))
                                (--map (cons (treemacs-workspace->name it) it))))
-              (name (completing-read "Switch to: " workspaces nil :require-match)))
+              (name (completing-read
+                     "Switch to: "
+                     (treemacs--pre-sorted-list workspaces)
+                     nil :require-match)))
          (setf new-workspace (cdr (--first (string= (car it) name) workspaces))))))
      (setf (treemacs-current-workspace) new-workspace)
      (treemacs--invalidate-buffer-project-cache)
@@ -655,8 +662,12 @@ Return values may be as follows:
      (treemacs-return
       `(success ,new-workspace)))))
 
-(defun treemacs-do-rename-workspace ()
+(defun treemacs-do-rename-workspace (&optional workspace new-name)
   "Rename a workspace.
+
+Takes either a WORKSPACE and a NEW-NAME as arguments or reads them
+interactively.
+
 Return values may be as follows:
 
 * If the given name is invalid:
@@ -667,19 +678,23 @@ Return values may be as follows:
   - the old-name
   - the renamed workspace"
   (treemacs-block
-   (let* ((current-ws (treemacs-current-workspace))
-          (old-name (treemacs-workspace->name current-ws))
-          (name-map (-> (--map (cons (treemacs-workspace->name it) it)  treemacs--workspaces)
-                        (sort (lambda (n _) (string= (car n) old-name)))))
-          (str-to-rename (completing-read "Rename: " name-map))
-          (ws-to-rename (cdr (assoc str-to-rename name-map)))
-          (new-name (treemacs--read-string "New name: ")))
+   (let ((old-name))
+     (unless workspace
+       (let* ((current-ws (treemacs-current-workspace))
+              (old-name (treemacs-workspace->name current-ws))
+              (name-map (-> (--map (cons (treemacs-workspace->name it) it)  treemacs--workspaces)
+                            (sort (lambda (n _) (string= (car n) old-name)))))
+              (str-to-rename (completing-read "Rename: " name-map)))
+         (setf workspace (cdr (assoc str-to-rename name-map)))))
+     (setf old-name (treemacs-workspace->name workspace))
+     (unless new-name
+       (setf new-name (treemacs--read-string "New name: ")))
      (treemacs-return-if (treemacs--is-name-invalid? new-name)
        `(invalid-name ,new-name))
-     (setf (treemacs-workspace->name ws-to-rename) new-name)
+     (setf (treemacs-workspace->name workspace) new-name)
      (treemacs--persist)
-     (run-hook-with-args 'treemacs-rename-workspace-functions ws-to-rename old-name)
-     `(success ,old-name ,ws-to-rename))))
+     (run-hook-with-args 'treemacs-rename-workspace-functions workspace old-name)
+     `(success ,old-name ,workspace))))
 
 (defun treemacs--is-name-invalid? (name)
   "Validate the NAME of a project or workspace.
@@ -708,13 +723,17 @@ PROJECT: Project Struct"
   (interactive)
   (save-excursion
     (goto-char (treemacs-project->position project))
-    (let* ((start (point-at-bol))
+    (let* ((start (line-beginning-position))
            (next  (treemacs--next-non-child-button (treemacs-project->position project)))
-           (end   (if next (-> next (treemacs-button-start) (previous-button) (treemacs-button-end)) (point-max))))
+           (end   (if next
+                      (-> next (treemacs-button-start) (previous-button) (treemacs-button-end))
+                    ;; final position minus the final newline
+                    (1- (point-max)))))
       (cons start end))))
 
 (defun treemacs--consolidate-projects ()
   "Correct treemacs buffers' content after the workspace was edited."
+  (treemacs--invalidate-buffer-project-cache)
   (treemacs-run-in-every-buffer
    (let* ((current-file (--when-let (treemacs-current-button) (treemacs--nearest-path it)))
           (current-workspace (treemacs-current-workspace))
@@ -746,7 +765,7 @@ PROJECT: Project Struct"
          (ht-remove! treemacs-dom (treemacs-project->path project-in-buffer))
          (setf projects-in-buffer (delete project-in-buffer projects-in-buffer))))
      (treemacs-with-writable-buffer
-      (treemacs--forget-last-highlight)
+      (treemacs--reset-dom)
       ;; delete everything's that's visible and render it again - the order of projects could
       ;; have been changed
       (erase-buffer)
@@ -781,7 +800,7 @@ PROJECT: Project Struct"
       (_
        (goto-char 0)
        (search-forward-regexp (rx-to-string `(seq bol ,line eol)))))
-    (setf treemacs--org-err-ov (make-overlay (point-at-eol) (point-at-eol)))
+    (setf treemacs--org-err-ov (make-overlay (line-end-position) (line-end-position)))
     (overlay-put treemacs--org-err-ov 'after-string
                  (concat (propertize " ← " 'face 'error) message))
     (add-hook 'after-change-functions #'treemacs--org-edit-remove-validation-msg nil :local)))
@@ -801,14 +820,25 @@ PROJECT: Project Struct"
      (--when-let (funcall fun)
        (treemacs-return it)))))
 
-(defun treemacs--select-workspace-by-name (&optional name)
-  "Interactively select the workspace with the given NAME."
+(defun treemacs--find-workspace-by-name (name)
+  "Find a workspace with the given NAME.
+Returns nil when there is no match."
   (treemacs--maybe-load-workspaces)
-  (-let [name (or name
-                   (completing-read
-                    "Workspace: "
-                    (->> treemacs--workspaces
-                         (--map (cons (treemacs-workspace->name it) it)))))]
+  (--first (string= name (treemacs-workspace->name it))
+           treemacs--workspaces))
+
+(defun treemacs--select-workspace-by-name ()
+  "Interactively select the workspace.
+Selection is based on the list of names of all workspaces and still happens
+when there is only one workspace."
+  (treemacs--maybe-load-workspaces)
+  (let (name)
+    (while (or (null name) (string= "" name))
+      (setf name (completing-read
+                  "Workspace: "
+                  (->> treemacs--workspaces
+                       (--map (cons (treemacs-workspace->name it) it)))
+                  nil :require-match)))
     (--first (string= name (treemacs-workspace->name it))
              treemacs--workspaces)))
 
