@@ -1,11 +1,11 @@
 ;;; treemacs.el --- A tree style file explorer package -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; Author: Alexander Miller <alexanderm@web.de>
 ;; Package-Requires: ((emacs "26.1") (cl-lib "0.5") (dash "2.11.0") (s "1.12.0") (ace-window "0.9.0") (pfuture "1.7") (hydra "0.13.2") (ht "2.2") (cfrs "1.3.2"))
 ;; Homepage: https://github.com/Alexander-Miller/treemacs
-;; Version: 2.9.5
+;; Version: 3.2
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -47,11 +47,11 @@
 (require 'treemacs-workspaces)
 (require 'treemacs-fringe-indicator)
 (require 'treemacs-header-line)
-(require 'treemacs-extensions)
+(require 'treemacs-annotations)
 
 (defconst treemacs-version
   (eval-when-compile
-    (format "v2.9.5 (installed %s) @ Emacs %s"
+    (format "v3.2 (installed %s) @ Emacs %s"
             (format-time-string "%Y.%m.%d" (current-time))
             emacs-version)))
 
@@ -68,29 +68,60 @@
   treemacs-version)
 
 ;;;###autoload
-(defun treemacs ()
+(defun treemacs (&optional arg)
   "Initialise or toggle treemacs.
-* If the treemacs window is visible hide it.
-* If a treemacs buffer exists, but is not visible show it.
-* If no treemacs buffer exists for the current frame create and show it.
-* If the workspace is empty additionally ask for the root path of the first
-  project to add."
-  (interactive)
+- If the treemacs window is visible hide it.
+- If a treemacs buffer exists, but is not visible show it.
+- If no treemacs buffer exists for the current frame create and show it.
+- If the workspace is empty additionally ask for the root path of the first
+  project to add.
+- With a prefix ARG launch treemacs and force it to select a workspace"
+  (interactive "P")
   (pcase (treemacs-current-visibility)
+    ((guard arg)
+     (treemacs-do-switch-workspace (treemacs--select-workspace-by-name))
+     (treemacs-select-window))
     ('visible (delete-window (treemacs-get-local-window)))
     ('exists  (treemacs-select-window))
     ('none    (treemacs--init))))
 
+;;;###autoload
+(defun treemacs-select-directory ()
+  "Select a directory to open in treemacs.
+This command will open *just* the selected directory in treemacs.  If there are
+other projects in the workspace they will be removed.
+
+To *add* a project to the current workspace use
+`treemacs-add-project-to-workspace' or
+`treemacs-add-and-display-current-project' instead."
+  (interactive)
+  (treemacs-block
+   (let* ((path (-> "Directory: "
+                    (read-directory-name)
+                    (treemacs-canonical-path)))
+          (name (treemacs--filename path))
+          (ws (treemacs-current-workspace)))
+     (treemacs-return-if
+         (and (= 1 (length (treemacs-workspace->projects ws)))
+              (string= path (-> ws
+                                (treemacs-workspace->projects)
+                                (car)
+                                (treemacs-project->path))))
+       (treemacs-select-window))
+     (treemacs--show-single-project path name)
+     (treemacs-pulse-on-success "Now showing %s"
+       (propertize path 'face 'font-lock-string-face)))))
+
 ;;;###autoload
 (defun treemacs-find-file (&optional arg)
   "Find and focus the current file in the treemacs window.
-If the current buffer has visits no file or with a prefix ARG ask for the
+If the current buffer visits no file or with a prefix ARG ask for the
 file instead.
 Will show/create a treemacs buffers if it is not visible/does not exist.
 For the most part only useful when `treemacs-follow-mode' is not active."
   (interactive "P")
-  (-let ((path (unless arg (buffer-file-name (current-buffer))))
-         (manually-entered nil))
+  (let ((path (unless arg (buffer-file-name (current-buffer))))
+        (manually-entered nil))
     (unless path
       (setq manually-entered t
             path (->> (--if-let (treemacs-current-button) (treemacs--nearest-path it))
@@ -141,24 +172,55 @@ visiting a file or Emacs cannot find any tags for the current file."
      (treemacs--do-follow-tag index treemacs-window buffer-file project))))
 
 ;;;###autoload
-(defun treemacs-select-window ()
+(defun treemacs-start-on-boot (&optional focus-treemacs)
+  "Initialiser specifically to start treemacs as part of your init file.
+
+Ensures that all visual elements are present which might otherwise be missing
+because their setup requires an interactive command or a post-command hook.
+
+FOCUS-TREEMACS indicates whether the treemacs window should be selected."
+  (-let [initial-window (selected-window)]
+    (treemacs)
+    (hl-line-highlight)
+    (redisplay)
+    (unless focus-treemacs (select-window initial-window))))
+
+;;;###autoload
+(defun treemacs-select-window (&optional arg)
   "Select the treemacs window if it is visible.
 Bring it to the foreground if it is not visible.
 Initialise a new treemacs buffer as calling `treemacs' would if there is no
 treemacs buffer for this frame.
-Jump back to the previously used window if point is already in treemacs."
-  (interactive)
+
+In case treemacs is already selected behaviour will depend on
+`treemacs-select-when-already-in-treemacs'.
+
+A non-nil prefix ARG will also force a workspace switch."
+  (interactive "P")
   (pcase (treemacs-current-visibility)
+    ((guard arg)
+     (treemacs-do-switch-workspace (treemacs--select-workspace-by-name))
+     (treemacs-select-window))
     ('exists  (treemacs--select-not-visible-window))
     ('none    (treemacs--init))
     ('visible
      (if (not (eq treemacs--in-this-buffer t))
          (treemacs--select-visible-window)
        (pcase-exhaustive treemacs-select-when-already-in-treemacs
-           ('stay
-            (ignore))
-           ('move-back
-            (select-window (get-mru-window (selected-frame) nil :not-selected))))))))
+         ('stay
+          (ignore))
+         ('close
+          (treemacs-quit))
+         ('goto-next
+          (treemacs--jump-to-next-treemacs-window))
+         ('next-or-back
+          (or
+           (treemacs--jump-to-next-treemacs-window)
+           (-if-let (mru-window (get-mru-window (selected-frame) nil :not-selected))
+               (select-window mru-window)
+             (treemacs-log-failure "get-mru-window could not find the last used window."))))
+         ('move-back
+          (select-window (get-mru-window (selected-frame) nil :not-selected))))))))
 
 ;;;###autoload
 (defun treemacs-show-changelog ()
@@ -191,14 +253,15 @@ Jump back to the previously used window if point is already in treemacs."
   (goto-char 0))
 
 ;;;###autoload
-(defun treemacs-display-current-project-exclusively ()
+(defun treemacs-add-and-display-current-project-exclusively ()
   "Display the current project, and *only* the current project.
 Like `treemacs-add-and-display-current-project' this will add the current
 project to treemacs based on either projectile, the built-in project.el, or the
 current working directory.
 
-However the 'exclusive' part means that it will make the current project the
-only project, all other projects *will be removed* from the current workspace."
+However the \\='exclusive\\=' part means that it will make the current project
+the only project, all other projects *will be removed* from the current
+workspace."
   (interactive)
   (treemacs-block
    (treemacs-unless-let (root (treemacs--find-current-user-project))
@@ -208,28 +271,19 @@ only project, all other projects *will be removed* from the current workspace."
             (name (treemacs--filename path))
             (ws (treemacs-current-workspace)))
        (treemacs-return-if
-           (and (= 1 (length (treemacs-workspace->projects ws)))
-                (treemacs-is-path path :in-workspace ws))
+           (-let [projects (treemacs-workspace->projects ws)]
+             (and (= 1 (length projects))
+                  (string=
+                   path
+                   (treemacs-project->path (car projects)))))
          (treemacs-select-window))
-       (if (treemacs-workspace->is-empty?)
-           (progn
-             (treemacs-do-add-project-to-workspace path name)
-             (treemacs-select-window)
-             (treemacs-pulse-on-success))
-         (setf (treemacs-workspace->projects ws)
-               (--filter (string= path (treemacs-project->path it))
-                         (treemacs-workspace->projects ws)))
-         (unless (treemacs-workspace->projects ws)
-           (let ((treemacs--no-messages t)
-                 (treemacs-pulse-on-success nil))
-             (treemacs-add-project-to-workspace path name)))
-         (treemacs-select-window)
-         (treemacs--consolidate-projects)
-         (goto-char 2)
-         (-let [btn (treemacs-current-button)]
-           (unless (treemacs-is-node-expanded? btn)
-             (treemacs--expand-root-node btn)))
-         (treemacs-pulse-on-success))))))
+       (treemacs--show-single-project path name)
+       (treemacs-pulse-on-success "Now showing %s"
+         (propertize path 'face 'font-lock-string-face))))))
+(define-obsolete-function-alias
+  'treemacs-display-current-project-exclusively
+  #'treemacs-add-and-display-current-project-exclusively
+  "v2.9")
 
 ;;;###autoload
 (defun treemacs-add-and-display-current-project ()
