@@ -1,6 +1,6 @@
 ;;; treemacs-evil.el --- Evil mode integration for treemacs -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; Author: Alexander Miller <alexanderm@web.de>
 ;; Package-Requires: ((emacs "26.1") (evil "1.2.12") (treemacs "0.0"))
@@ -32,8 +32,17 @@
   treemacs-common-helpful-hydra
   treemacs-advanced-helpful-hydra)
 
+(treemacs-import-functions-from "treemacs-mouse-interface"
+  treemacs-dragleftclick-action
+  treemacs-leftclick-action)
+
+(treemacs-import-functions-from "treemacs-mouse-interface"
+  treemacs-copy-file)
+
 (declare-function treemacs-add-bookmark "treemacs-bookmarks.el")
 
+(declare-function treemacs-git-commit-diff-mode "treemacs-commit-diff-mode.el")
+
 (evil-define-state treemacs
   "Treemacs state"
   :cursor '(bar . 0)
@@ -82,6 +91,8 @@ Then call ORIG-FUN with its ARGS and reopen treemacs if it was open before."
 (define-key evil-treemacs-state-map (kbd "tf")  #'treemacs-follow-mode)
 (define-key evil-treemacs-state-map (kbd "ta")  #'treemacs-filewatch-mode)
 (define-key evil-treemacs-state-map (kbd "tg")  #'treemacs-git-mode)
+(define-key evil-treemacs-state-map (kbd "tc")  #'treemacs-indicate-top-scroll-mode)
+(define-key evil-treemacs-state-map (kbd "td")  #'treemacs-git-commit-diff-mode)
 (define-key evil-treemacs-state-map (kbd "tn")  #'treemacs-indent-guide-mode)
 (define-key evil-treemacs-state-map (kbd "w")   #'treemacs-set-width)
 (define-key evil-treemacs-state-map (kbd ">")   #'treemacs-increase-width)
@@ -99,13 +110,17 @@ Then call ORIG-FUN with its ARGS and reopen treemacs if it was open before."
 (evil-define-key 'treemacs treemacs-mode-map (kbd "yp")     #'treemacs-copy-project-path-at-point)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "ya")     #'treemacs-copy-absolute-path-at-point)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "yr")     #'treemacs-copy-relative-path-at-point)
+(evil-define-key 'treemacs treemacs-mode-map (kbd "yn")     #'treemacs-copy-filename-at-point)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "yf")     #'treemacs-copy-file)
+(evil-define-key 'treemacs treemacs-mode-map (kbd "yv")     #'treemacs-paste-dir-at-point-to-minibuffer)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "gr")     #'treemacs-refresh)
 (evil-define-key 'treemacs treemacs-mode-map [down-mouse-1] #'treemacs-leftclick-action)
 (evil-define-key 'treemacs treemacs-mode-map [drag-mouse-1] #'treemacs-dragleftclick-action)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "h")      #'treemacs-COLLAPSE-action)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "RET")    #'treemacs-RET-action)
 (evil-define-key 'treemacs treemacs-mode-map (kbd "l")      #'treemacs-RET-action)
+(unless (window-system)
+  (evil-define-key 'treemacs treemacs-mode-map [C-i] #'treemacs-TAB-action))
 
 (provide 'treemacs-evil)
 
