@@ -1,6 +1,6 @@
 ;;; treemacs-test.el --- Tests for treemacs -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021 Alexander Miller
+;; Copyright (C) 2024 Alexander Miller
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 (require 'treemacs-tags)
 (require 'treemacs-tag-follow-mode)
 (require 'treemacs-mouse-interface)
+(require 'treemacs-file-management)
 (require 'org)
 (require 'buttercup)
 
@@ -77,6 +78,38 @@
             (parent "~/A/prefix2"))
         (expect (treemacs-is-path path :in parent) :not :to-be-truthy))))
 
+  (describe ":directly-in matcher"
+
+    (it "identifies direct parent"
+      (let ((path "~/A/B/c")
+            (parent "~/A/B"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be-truthy)))
+
+    (it "rejects indirect parent"
+      (let ((path "~/A/B/c")
+            (parent "~/A"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be nil)))
+
+    (it "rejects non-parent"
+      (let ((path "~/A/B/c")
+            (parent "x"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be nil)))
+
+    (it "rejects non-parent with same length as parent"
+      (let ((path "~/A/B/c")
+            (parent "~/A/X"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be nil)))
+
+    (it "rejects non-parent with similar preix"
+      (let ((path "~/A/prefix1")
+            (parent "~/A/prefix2"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be nil)))
+
+    (it "rejects shorter path than parent"
+      (let ((path "~/A")
+            (parent "~/A/B/C"))
+        (expect (treemacs-is-path path :directly-in parent) :to-be nil))))
+
   (describe ":in-project matcher"
 
     (it "Identifies that a path is in a project"
@@ -123,7 +156,11 @@
         (expect (treemacs--reject-ignored-files "foo.el") :to-be t))
 
       (it "Accepts directory"
-        (expect (treemacs--reject-ignored-files "~/A/B/C/") :to-be t)))
+        (expect (treemacs--reject-ignored-files "~/A/B/C/") :to-be t))
+
+      (it "Accepts .git when it is not hidden"
+        (-let [treemacs-hide-dot-git-directory nil]
+          (expect (treemacs--reject-ignored-files "~/A/B/C/.git") :to-be t))))
 
     (describe "Rejecting"
 
@@ -145,9 +182,6 @@
       (it "Rejects flycheck's temp file"
         (expect (treemacs--reject-ignored-files "~/A/B/C/flycheck_foo.el") :to-be nil))
 
-      (it "Rejects .git"
-        (expect (treemacs--reject-ignored-files "~/A/B/C/.git") :to-be nil))
-
       (it "Rejects dot"
         (expect (treemacs--reject-ignored-files ".") :to-be nil))
 
@@ -158,7 +192,7 @@
 
   (let ((treemacs-ignored-file-predicates (default-value 'treemacs-ignored-file-predicates)))
 
-    (describe "Acceptions"
+    (describe "Accepting"
 
       (it "Accepts common absolute path"
         (expect (treemacs--reject-ignored-and-dotfiles "~/A/B/C/foo.el") :to-be t))
@@ -169,7 +203,7 @@
       (it "Accepts directory"
         (expect (treemacs--reject-ignored-and-dotfiles "~/A/B/C/") :to-be t)))
 
-    (describe "Rejections"
+    (describe "Rejecting"
 
       (it "Fails on nil input"
         (expect (treemacs--reject-ignored-and-dotfiles nil) :to-throw))
@@ -193,7 +227,7 @@
         (expect (treemacs--reject-ignored-and-dotfiles "~/A/B/C/.foo.el") :to-be nil))
 
       (it "Rejects .git"
-        (expect (treemacs--reject-ignored-files "~/A/B/C/.git") :to-be nil))
+        (expect (treemacs--reject-ignored-and-dotfiles "~/A/B/C/.git") :to-be nil))
 
       (it "Rejects dot"
         (expect (treemacs--reject-ignored-and-dotfiles ".") :to-be nil))
@@ -338,8 +372,8 @@
 
 (describe "treemacs--parent"
 
-  (it "Fails on nil input"
-    (expect (treemacs--parent nil) :to-throw))
+  (it "Does not fail for nil input"
+    (expect (treemacs--parent nil) :to-be nil))
 
   (it "Returns nil when input is empty"
     (expect (treemacs--parent "") :to-be nil))
@@ -354,21 +388,10 @@
     (expect (treemacs--parent "/") :to-equal "/"))
 
   (it "Returns parent of root-level extension node."
-    (expect (treemacs--parent '(:custom "a" "b")) :to-equal '(:custom "a")))
+    (expect (treemacs--parent '(a b)) :to-equal '(a)))
 
-  (it "Returns directory extension of extension sub-item node."
-    (expect (treemacs--parent '("/test1" "a" "b")) :to-equal '("/test1" "a")))
-
-  (it "Returns directory of a directory extension node."
-    (expect (treemacs--parent '("/test1" "a")) :to-equal "/test1"))
-
-  (it "Returns project extension of a project sub-item node."
-    (let ((p (treemacs-project->create! :path "/A" :path-status 'local-readable)))
-      (expect (treemacs--parent (list p "a" "b")) :to-equal (list p "a"))))
-
-  (it "Returns project of a project extension node."
-    (let ((p (treemacs-project->create! :path "/A" :path-status 'local-readable)))
-      (expect (treemacs--parent (list p "a")) :to-equal "/A"))))
+  (it "Returns parent of directory extension node."
+    (expect (treemacs--parent '("/test1" "a" "b")) :to-equal '("/test1" "a"))))
 
 (describe "treemacs--get-or-parse-git-result"
 
@@ -919,20 +942,49 @@
   (describe "Successes"
 
     (it "Succeeds on correctly formed input"
-      (-let [lines '("* W1" "** P1" " - path :: a" "** P2" "- path :: b" "* W2" "** P3" " - path :: c")]
+      (-let [lines '("* W1"
+                     "** P1"
+                     " - path :: a"
+                     "** P2"
+                     "- path :: b"
+                     "* W2"
+                     "** P3"
+                     " - path :: c")]
         (expect (treemacs--validate-persist-lines lines) :to-be 'success)))
 
     (it "Succeeds with the same path in multiple workspaces"
-      (-let [lines '("* W1" "** P1" " - path :: /A/B" "* W2" "** P2" " - path :: /A/B")]
+      (-let [lines '("* W1"
+                     "** P1"
+                     " - path :: /A/B"
+                     "* W2"
+                     "** P2"
+                     " - path :: /A/B")]
         (expect (treemacs--validate-persist-lines lines) :to-be 'success)))
 
     (it "Succeeds with non-connectable remotes"
       (let* ((treemacs--org-edit-buffer-name (buffer-name))
-             (lines '("* W1" "** P1" " - path :: /ftp:anonymous@ftp.invalid:/test-path")))
+             (lines '("* W1"
+                      "** P1"
+                      " - path :: /ftp:anonymous@ftp.invalid:/test-path")))
         (expect (treemacs--validate-persist-lines lines) :to-be 'success)))
 
     (it "Succeeds with disabled projects"
-      (-let [lines '("* W1" "** COMMENT P1" " - path :: a" "** P2" " - path :: b" "** COMMENT P3" " - path :: c")]
+      (-let [lines '("* W1"
+                     "** COMMENT P1"
+                     " - path :: a"
+                     "** P2"
+                     " - path :: b"
+                     "** COMMENT P3"
+                     " - path :: c")]
+        (expect (treemacs--validate-persist-lines lines) :to-be 'success)))
+
+    (it "Succeeds with disabled workspaces"
+      (-let [lines '("* COMMENT W1"
+                     "** P1"
+                     " - path :: a"
+                     "* W2"
+                     "** P2"
+                     " - path :: b")]
         (expect (treemacs--validate-persist-lines lines) :to-be 'success))))
 
   (describe "Errors"
@@ -943,27 +995,34 @@
                 :to-equal '(error "X" "First item must be a workspace name"))))
 
     (it "Fails when line after workspace name is not a project name"
-      (-let [lines '("* X" "Y")]
+      (-let [lines '("* X"
+                     "Y")]
         (expect (treemacs--validate-persist-lines lines)
-                :to-equal '(error  "Y" "Workspace name must be followed by project name"))))
+                :to-equal '(error "Y" "Workspace name must be followed by project name"))))
 
     (it "Fails when line after project name is not a property"
-      (-let [lines '("* X" "** Y" "Z")]
+      (-let [lines '("* X"
+                     "** Y"
+                     "Z")]
         (expect (treemacs--validate-persist-lines lines)
-                :to-equal '(error  "** Y" "Project name must be followed by path declaration"))))
+                :to-equal '(error "** Y" "Project name must be followed by path declaration"))))
 
     (it "Fails when line after path is not a project or workspace"
-      (-let [lines '("* X" "** Y" " - path :: Z" "A")]
+      (-let [lines '("* X"
+                     "** Y"
+                     " - path :: Z"
+                     "A")]
         (expect (treemacs--validate-persist-lines lines)
                 :to-equal '(error " - path :: Z" "Path property must be followed by the next workspace or project"))))
 
-    (it "Fails when line end at workspace name"
+    (it "Fails when lines end at workspace name"
       (-let [lines '("* X")]
         (expect (treemacs--validate-persist-lines lines)
                 :to-equal '(error "* X" "Cannot end with a project or workspace name"))))
 
-    (it "Fails when line end at project name"
-      (-let [lines '("* X" "** X")]
+    (it "Fails when lines end at project name"
+      (-let [lines '("* X"
+                     "** X")]
         (expect (treemacs--validate-persist-lines lines)
                 :to-equal '(error "** X" "Cannot end with a project or workspace name"))))
 
@@ -982,9 +1041,25 @@
                 '(error " - path :: /A/B/C/D" "Path '/A/B/C/D' appears in the workspace more than once."))))
 
     (it "Fails when all projects are disabled"
-      (-let [lines '("* W1" "** COMMENT P1" " - path :: a" "** COMMENT P2" " - path :: b" "** COMMENT P3" " - path :: c")]
+      (-let [lines '("* W1"
+                     "** COMMENT P1"
+                     " - path :: a"
+                     "** COMMENT P2"
+                     " - path :: b"
+                     "** COMMENT P3"
+                     " - path :: c")]
         (expect (treemacs--validate-persist-lines lines)
-                :to-equal '(error " - path :: c" "Workspace must contain at least 1 project that is not disabled."))))))
+                :to-equal '(error " - path :: c" "Workspace must contain at least 1 project that is not disabled."))))
+
+    (it "Fails when all workspaces are disabled"
+      (-let [lines '("* COMMENT W1"
+                     "** P1"
+                     " - path :: a"
+                     "* COMMENT W2"
+                     "** P2"
+                     " - path :: b")]
+        (expect (treemacs--validate-persist-lines lines)
+                :to-equal '(error " - path :: b" "There must be at least 1 worspace that is not disabled."))))))
 
 (describe "treemacs--read-persist-lines"
 
@@ -1006,15 +1081,25 @@
     (spy-on #'treemacs--get-path-status :and-return-value 'local-readable))
 
   (it "Reads workspaces correctly"
-    (let* ((list '("* WS 1" "** P1" " - path :: /a" "** P2" " - path :: /b"
-                   "* WS 2" "** P3" " - path :: /c" "** P4" " - path :: /d"))
+    (let* ((list '("* WS 1"
+                   "** P1"
+                   " - path :: /a"
+                   "** P2" " - path :: /b"
+                   "* WS 2"
+                   "** P3"
+                   " - path :: /c"
+                   "** P4"
+                   " - path :: /d"))
            (iter (treemacs-iter->create! :list list))
-           (result (treemacs--read-workspaces iter)))
+           (result (treemacs--read-workspaces iter))
+           (disabled-workspaces (car result))
+           (workspaces (cadr result)))
 
-      (expect (length result) :to-be 2)
+      (expect (length disabled-workspaces) :to-be 0)
+      (expect (length workspaces) :to-be 2)
 
-      (let* ((ws1 (car result))
-             (ws2 (cadr result))
+      (let* ((ws1 (car workspaces))
+             (ws2 (cadr workspaces))
              (ws1-projects (treemacs-workspace->projects ws1))
              (ws2-projects (treemacs-workspace->projects ws2)))
 
@@ -1037,14 +1122,42 @@
           (expect (treemacs-project->path pr3) :to-equal "/c")
           (expect (treemacs-project->path pr4) :to-equal "/d")))))
 
-  (it "Reads disabled projects"
-    (let* ((list '("* WS 1" "** COMMENT P1" " - path :: /a"))
+  (it "Reads disbled workspaces correctly"
+    (let* ((list '("* COMMENT WS 1"
+                   "** P1"
+                   " - path :: /a"))
            (iter (treemacs-iter->create! :list list))
-           (result (treemacs--read-workspaces iter)))
+           (result (treemacs--read-workspaces iter))
+           (disabled-workspaces (car result))
+           (workspaces (cadr result)))
 
-      (expect (length result) :to-be 1)
+      (expect (length disabled-workspaces) :to-be 1)
+      (expect (length workspaces) :to-be 0)
 
-      (-let [project (-> result (car) (treemacs-workspace->projects) (car))]
+      (let* ((ws1 (car disabled-workspaces))
+             (ws1-projects (treemacs-workspace->projects ws1)))
+
+        (expect (treemacs-workspace->name ws1) :to-equal "WS 1")
+        (expect (treemacs-workspace->is-disabled? ws1) :to-be t)
+        (expect (length ws1-projects) :to-be 1)
+
+        (let ((pr1 (car ws1-projects)))
+          (expect (treemacs-project->name pr1) :to-equal "P1")
+          (expect (treemacs-project->path pr1) :to-equal "/a")))))
+
+  (it "Reads disabled projects"
+    (let* ((list '("* WS 1"
+                   "** COMMENT P1"
+                   " - path :: /a"))
+           (iter (treemacs-iter->create! :list list))
+           (result (treemacs--read-workspaces iter))
+           (workspaces (cadr result))
+           (disabled-workspaces (car result)))
+
+      (expect (length workspaces) :to-be 1)
+      (expect (length disabled-workspaces) :to-be 0)
+
+      (-let [project (-> workspaces (car) (treemacs-workspace->projects) (car))]
         (expect (treemacs-project->name project) :to-equal "P1")
         (expect (treemacs-project->is-disabled? project) :to-be t)))))
 
@@ -1073,36 +1186,10 @@
       (dolist (status '(local-unreadable remote-readable remote-unreadable remote-disconnected extension))
         (expect (-> treemacs-dir
                     (treemacs-join-path "test")
-                    (treemacs--collapsed-dirs-process (treemacs-project->create! :name "P" :path treemacs-dir :path-status status)))
+                    (treemacs--flattened-dirs-process (treemacs-project->create! :name "P" :path treemacs-dir :path-status status)))
                 :to-equal
                 nil)))))
 
-(describe "treemacs--parse-collapsed-dirs"
-
-  (it "Finds dirs to flatten in test directory"
-    (-let [treemacs-collapse-dirs 3]
-      (expect (-> treemacs-dir
-                  (treemacs-join-path "test")
-                  (treemacs--collapsed-dirs-process
-                   (treemacs-project->create!
-                    :name "P"
-                    :path treemacs-dir
-                    :path-status 'local-readable))
-                  (treemacs--parse-collapsed-dirs))
-              :to-equal
-              `(("/testdir2/testdir3"
-                 ,(treemacs-join-path treemacs-dir "test/testdir1")
-                 ,(treemacs-join-path treemacs-dir "test/testdir1/testdir2")
-                 ,(treemacs-join-path treemacs-dir "test/testdir1/testdir2/testdir3"))))))
-
-  (it "Returns nil when there is nothing to flatten"
-    (-let [treemacs-collapse-dirs 3]
-      (expect (-> treemacs-dir
-                  (treemacs-join-path "test/testdir1/testdir2")
-                  (treemacs--collapsed-dirs-process (treemacs-project->create! :name "P" :path treemacs-dir :path-status 'local-readable))
-                  (treemacs--parse-collapsed-dirs))
-              :to-be nil))))
-
 (describe "treemacs--remove-trailing-newline"
 
   (it "Fails on nil input"
@@ -1254,17 +1341,17 @@ EXPECTED-3 is the expected expansion of the \"file.txt\" button."
   (it "Finds nothing for node without direct children"
     (with-temp-buffer
       (let* ((root1 (progn
-                      (insert (propertize "Root1" 'button t 'category t :depth 1))
+                      (insert (propertize "Root1" 'button t 'category 'treemacs-button :depth 1))
                       (beginning-of-line)
                       (point-marker)))
              (input (progn
                       (end-of-line)
-                      (insert "\n" (propertize " Input" 'button t 'category t :depth 2 :parent root1))
+                      (insert "\n" (propertize " Input" 'button t 'category 'treemacs-button :depth 2 :parent root1))
                       (beginning-of-line)
                       (point-marker)))
              (_root2 (progn
                        (end-of-line)
-                       (insert "\n" (propertize "Root2" 'button t 'category t :depth 1))
+                       (insert "\n" (propertize "Root2" 'button t 'category 'treemacs-button :depth 1))
                        (beginning-of-line)
                        (point-marker))) )
         (-let [result (treemacs-collect-child-nodes input)]
@@ -1273,27 +1360,27 @@ EXPECTED-3 is the expected expansion of the \"file.txt\" button."
   (it "Finds only direct children"
     (with-temp-buffer
       (let* ((root1 (progn
-                      (insert (propertize "Root1" 'button t 'category t :depth 1))
+                      (insert (propertize "Root1" 'button t 'category 'treemacs-button :depth 1))
                       (beginning-of-line)
                       (point-marker)))
              (input (progn
                       (end-of-line)
-                      (insert "\n" (propertize " Input" 'button t 'category t :depth 2 :parent root1))
+                      (insert "\n" (propertize " Input" 'button t 'category 'treemacs-button :depth 2 :parent root1))
                       (beginning-of-line)
                       (point-marker)))
              (child1 (progn
                        (end-of-line)
-                       (insert "\n" (propertize "  Child1" 'button t 'category t :depth 3 :parent input))
+                       (insert "\n" (propertize "  Child1" 'button t 'category 'treemacs-button :depth 3 :parent input))
                        (beginning-of-line)
                        (point-marker)))
              (_grand-child (progn
                              (end-of-line)
-                             (insert "\n" (propertize "   Grand Child" 'button t 'category t :depth 4 :parent child1))
+                             (insert "\n" (propertize "   Grand Child" 'button t 'category 'treemacs-button :depth 4 :parent child1))
                              (beginning-of-line)
                              (point-marker)))
              (_child2 (progn
                         (end-of-line)
-                        (insert "\n" (propertize "  Child2" 'button t 'category t :depth 3 :parent input))
+                        (insert "\n" (propertize "  Child2" 'button t 'category 'treemacs-button :depth 3 :parent input))
                         (beginning-of-line)
                         (point-marker))) )
         (-let [result (-map #'treemacs--get-label-of (treemacs-collect-child-nodes input))]
@@ -1636,6 +1723,364 @@ EXPECTED-3 is the expected expansion of the \"file.txt\" button."
           (expect (treemacs-do-switch-workspace ws2)
                   :to-equal `(success ,ws2)))))))
 
+(describe "treemacs--select-file-from-current-btn"
+  :var (btn)
+
+  (before-each
+    (fset
+     'make-btn
+     (lambda (&rest props)
+       (insert "abc")
+       (goto-char 0)
+       (put-text-property 1 3 :path "/a/b/c")
+       (put-text-property 1 3 :button t)
+       (when props
+         (apply #'put-text-property 1 3 props))
+       (setf btn (point-marker))))
+    (spy-on #'treemacs-current-button :and-return-value btn)
+    (spy-on #'file-directory-p :and-return-value nil)
+    (spy-on #'file-regular-p :and-return-value nil))
+
+  (it "returns interactively chosen path for flattened nodes"
+    (with-temp-buffer
+      (with-no-warnings (make-btn :collapsed '(2 "/a" "/a/b" "/a/b/c")))
+      (spy-on #'completing-read :and-return-value "/a/b")
+      (expect (treemacs--select-file-from-btn btn "prompt")
+              :to-equal "/a/b")))
+
+  (it "returns path when it is a directory"
+    (with-temp-buffer
+      (with-no-warnings (make-btn))
+      (spy-on #'file-directory-p :and-return-value t)
+      (expect (treemacs--select-file-from-btn btn "prompt")
+              :to-equal "/a/b/c")))
+
+  (it "returns parent path when path is file and dir-only is set"
+    (with-temp-buffer
+      (with-no-warnings (make-btn))
+      (spy-on #'file-regular-p :and-return-value t)
+      (expect (treemacs--select-file-from-btn btn "prompt" :dir-only)
+              :to-equal "/a/b")))
+
+  (it "returns path when it is a file and dir-only is not set"
+    (with-temp-buffer
+      (with-no-warnings (make-btn))
+      (spy-on #'file-regular-p :and-return-value t)
+      (expect (treemacs--select-file-from-btn btn "prompt" nil)
+              :to-equal "/a/b/c")))
+
+  (it "returns HOME when path is not a string"
+    (with-temp-buffer
+      (with-no-warnings (make-btn :path '(a b c)))
+      (spy-on #'file-regular-p :and-return-value t)
+      (expect (treemacs--select-file-from-btn btn "prompt")
+              :to-equal (expand-file-name "~")))) )
+
+(describe "treemacs--select-workspace-by-name"
+  (before-each
+    (spy-on #'treemacs--maybe-load-workspaces :and-return-value nil))
+
+  (it "returns match for only 1 workspace"
+    (spy-on #'completing-read :and-return-value "WS")
+    (let* ((ws (treemacs-workspace->create! :name "WS"))
+           (treemacs--workspaces (list ws)))
+      (expect (treemacs--select-workspace-by-name) :to-be ws)))
+
+  (it "returns match for more than 1 workspace"
+    (spy-on #'completing-read :and-return-value "WS2")
+    (let* ((ws1 (treemacs-workspace->create! :name "WS1"))
+           (ws2 (treemacs-workspace->create! :name "WS2"))
+           (treemacs--workspaces (list ws1 ws2)))
+      (expect (treemacs--select-workspace-by-name) :to-be ws2)))
+
+  (it "forces a name selection"
+    (spy-on #'completing-read :and-call-fake
+            (-let [c 0]
+              (lambda (&rest _)
+                (if (> c 3)
+                    "WS2"
+                  (cl-incf c)
+                  ""))))
+    (let* ((ws1 (treemacs-workspace->create! :name "WS1"))
+           (ws2 (treemacs-workspace->create! :name "WS2"))
+           (treemacs--workspaces (list ws1 ws2)))
+
+      (expect (treemacs--select-workspace-by-name) :to-be ws2))))
+
+(describe "treemacs--find-workspace-by-name"
+  (before-each
+    (spy-on #'treemacs--maybe-load-workspaces :and-return-value nil))
+
+  (it "returns match for only 1 workspace"
+    (let* ((ws (treemacs-workspace->create! :name "WS1"))
+           (treemacs--workspaces (list ws)))
+      (expect (treemacs--find-workspace-by-name "WS1") :to-be ws)))
+
+  (it "returns match for more than 1 workspace"
+    (let* ((ws1 (treemacs-workspace->create! :name "WS1"))
+           (ws2 (treemacs-workspace->create! :name "WS2"))
+           (treemacs--workspaces (list ws1 ws2)))
+      (expect (treemacs--find-workspace-by-name "WS2") :to-be ws2)))
+
+  (it "returns nil when there is no match"
+    (let* ((ws1 (treemacs-workspace->create! :name "WS1"))
+           (ws2 (treemacs-workspace->create! :name "WS2"))
+           (treemacs--workspaces (list ws1 ws2)))
+      (expect (treemacs--find-workspace-by-name "X") :to-be nil))))
+
+(describe "annotations"
+
+  (describe "cleanup"
+
+    (it "won't remove annotation if it has a face"
+      (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+        (treemacs-set-annotation-face "Path" 'face "Source")
+        (-let [ann (treemacs-get-annotation "Path")]
+          (treemacs--remove-annotation-if-empty ann "Path")
+          (expect (treemacs-get-annotation "Path") :to-be ann))))
+
+    (it "won't remove annotation if it has a git face"
+      (let* ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+             (ann (treemacs-annotation->create!)))
+        (ht-set! treemacs--annotation-store "Path" ann)
+        (setf (treemacs-annotation->git-face ann) 'face)
+        (treemacs--remove-annotation-if-empty ann "Path")
+        (expect (treemacs-get-annotation "Path") :to-be ann)))
+
+    (it "won't remove annotation if it has a suffix"
+      (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+        (treemacs-set-annotation-suffix "Path" "Suffix" "Source")
+        (-let [ann (treemacs-get-annotation "Path")]
+          (treemacs--remove-annotation-if-empty ann "Path")
+          (expect (treemacs-get-annotation "Path") :to-be ann))))
+
+    (it "removes empty annotation"
+      (let* ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+             (ann (treemacs-annotation->create!)))
+        (ht-set! treemacs--annotation-store "Path" ann)
+        (treemacs--remove-annotation-if-empty ann "Path")
+        (expect (treemacs-get-annotation "Path") :to-be nil))))
+
+  (describe "faces"
+
+    (describe "add"
+
+      (it "saves single value"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-face "Path" 'face "Source")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->face ann)
+                    :to-equal '(("Source" . face)))
+            (expect (treemacs-annotation->face-value ann)
+                    :to-equal '(face)))))
+
+      (it "saves multiple values"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-face "Path" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path" 'face2 "Source2")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->face ann)
+                    :to-equal '(("Source1" . face1) ("Source2" . face2)))
+            (expect (treemacs-annotation->face-value ann)
+                    :to-equal '(face1 face2)))))
+
+      (it "updates face for same source"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-face "Path" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path" 'face2 "Source1")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->face ann)
+                    :to-equal '(("Source1" . face2)))
+            (expect (treemacs-annotation->face-value ann)
+                    :to-equal '(face2)))))
+
+      (it "includes git face as last value element"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+              (ann (treemacs-annotation->create!)))
+          (ht-set! treemacs--annotation-store "Path" ann)
+          (setf (treemacs-annotation->git-face ann) 'git-face)
+          (treemacs-set-annotation-face "Path" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path" 'face2 "Source2")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->face ann)
+                    :to-equal '(("Source1" . face1) ("Source2" . face2)))
+            (expect (treemacs-annotation->face-value ann)
+                    :to-equal '(face1 face2 . git-face))))))
+
+    (describe "remove"
+
+      (it "does nothing when there is no face"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-remove-annotation-face "Path" "Source")
+          (expect (ht-size treemacs--annotation-store) :to-equal 0)))
+
+      (it "removes the last face"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+              (ann (treemacs-annotation->create!)))
+          (ht-set! treemacs--annotation-store "Path" ann)
+          (treemacs-set-annotation-face "Path" 'face "Source")
+          (treemacs-remove-annotation-face "Path" "Source")
+          (expect (ht-get treemacs--annotation-store "Path") :to-be ann)
+          (expect (treemacs-annotation->face ann) :to-be nil)
+          (expect (treemacs-annotation->face-value ann) :to-be nil)))
+
+      (it "sets face value to the git face after removing the last face"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+              (ann (treemacs-annotation->create!)))
+          (ht-set! treemacs--annotation-store "Path" ann)
+          (setf (treemacs-annotation->git-face ann) 'git-face)
+          (treemacs-set-annotation-face "Path" 'face "Source")
+          (treemacs-remove-annotation-face "Path" "Source")
+          (expect (ht-get treemacs--annotation-store "Path") :to-be ann)
+          (expect (treemacs-annotation->face-value ann) :to-equal 'git-face)))
+
+      (it "leaves faces from other sources"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+              (ann (treemacs-annotation->create!)))
+          (ht-set! treemacs--annotation-store "Path" ann)
+          (setf (treemacs-annotation->git-face ann) 'git-face)
+          (treemacs-set-annotation-face "Path" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path" 'face2 "Source2")
+          (treemacs-remove-annotation-face "Path" "Source1")
+          (expect (ht-get treemacs--annotation-store "Path") :to-be ann)
+          (expect (treemacs-annotation->face ann) :to-equal '(("Source2" . face2)))
+          (expect (treemacs-annotation->face-value ann) :to-equal '(face2 . git-face)))))
+
+    (describe "clear"
+
+      (it "does nothing when there are no annotations"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-clear-annotation-faces "Source")
+          (expect (ht-size treemacs--annotation-store) :to-be 0)))
+
+      (it "does not remove faces for a different source"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-face "Path" 'face "Source1")
+          (treemacs-clear-annotation-faces "Source2")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect ann :not :to-be nil)
+            (expect (treemacs-annotation->face ann) :to-equal '(("Source1" . face))))))
+
+      (it "removes empty annotations"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-face "Path" 'face "Source")
+          (treemacs-clear-annotation-faces "Source")
+          (expect (treemacs-get-annotation "Path") :to-be nil)))
+
+      (it "removes all faces for the given source"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-face "Path1" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path1" 'face2 "Source2")
+          (treemacs-set-annotation-face "Path2" 'face1 "Source1")
+          (treemacs-set-annotation-face "Path2" 'face3 "Source3")
+          (treemacs-clear-annotation-faces "Source1")
+          (let ((ann1 (treemacs-get-annotation "Path1"))
+                (ann2 (treemacs-get-annotation "Path2")))
+            (expect ann1 :not :to-be nil)
+            (expect ann2 :not :to-be nil)
+            (expect (treemacs-annotation->face ann1) :to-equal '(("Source2" . face2)))
+            (expect (treemacs-annotation->face ann2) :to-equal '(("Source3" . face3))))))))
+
+  (describe "suffixes"
+
+    (describe "add"
+
+      (it "saves single value"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-suffix "Path" "Suffix" "Source")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->suffix ann)
+                    :to-equal '(("Source" . "Suffix")))
+            (expect (treemacs-annotation->suffix-value ann)
+                    :to-equal "Suffix"))))
+
+      (it "saves multiple values"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-suffix "Path" "Suffix1" "Source1")
+          (treemacs-set-annotation-suffix "Path" "Suffix2" "Source2")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->suffix ann)
+                    :to-equal '(("Source1" . "Suffix1") ("Source2" . "Suffix2")))
+            (expect (substring-no-properties (treemacs-annotation->suffix-value ann))
+                    :to-equal "Suffix1 Suffix2"))))
+
+      (it "updates suffix for same source"
+        (-let [treemacs--annotation-store (make-hash-table :size 200 :test 'equal)]
+          (treemacs-set-annotation-suffix "Path" "Suffix1" "Source1")
+          (treemacs-set-annotation-suffix "Path" "Suffix2" "Source1")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect (treemacs-annotation->suffix ann)
+                    :to-equal '(("Source1" . "Suffix2")))
+            (expect (substring-no-properties (treemacs-annotation->suffix-value ann))
+                    :to-equal "Suffix2")))))
+
+    (describe "remove"
+
+      (it "does nothing when there is no suffix"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-remove-annotation-suffix "Path" "Source")
+          (expect (ht-size treemacs--annotation-store) :to-equal 0)))
+
+      (it "leaves suffixes from other sources"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal))
+              (ann (treemacs-annotation->create!)))
+          (ht-set! treemacs--annotation-store "Path" ann)
+          (treemacs-set-annotation-suffix "Path" "Suffix1" "Source1")
+          (treemacs-set-annotation-suffix "Path" "Suffix2" "Source2")
+          (treemacs-remove-annotation-suffix "Path" "Source1")
+          (expect (ht-get treemacs--annotation-store "Path") :to-be ann)
+          (expect (treemacs-annotation->suffix ann) :to-equal '(("Source2" . "Suffix2")))
+          (expect (treemacs-annotation->suffix-value ann) :to-equal "Suffix2"))))
+
+    (describe "clear"
+
+      (it "does nothing when there are no annotations"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-clear-annotation-faces "Source")
+          (expect (ht-size treemacs--annotation-store) :to-be 0)))
+
+      (it "does not remove suffixes for a different source"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-suffix "Path" "Suffix" "Source1")
+          (treemacs-clear-annotation-suffixes "Source2")
+          (-let [ann (treemacs-get-annotation "Path")]
+            (expect ann :not :to-be nil)
+            (expect (treemacs-annotation->suffix ann) :to-equal '(("Source1" . "Suffix"))))))
+
+      (it "removes empty annotations"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-suffix "Path" "Suffix" "Source")
+          (treemacs-clear-annotation-suffixes "Source")
+          (expect (treemacs-get-annotation "Path") :to-be nil)))
+
+      (it "removes all suffixes for the given source"
+        (let ((treemacs--annotation-store (make-hash-table :size 200 :test 'equal)))
+          (treemacs-set-annotation-suffix "Path1" "Suffix1" "Source1")
+          (treemacs-set-annotation-suffix "Path1" "Suffix2" "Source2")
+          (treemacs-set-annotation-suffix "Path2" "Suffix1" "Source1")
+          (treemacs-set-annotation-suffix "Path2" "Suffix3" "Source3")
+          (treemacs-clear-annotation-suffixes "Source1")
+          (let ((ann1 (treemacs-get-annotation "Path1"))
+                (ann2 (treemacs-get-annotation "Path2")))
+            (expect ann1 :not :to-be nil)
+            (expect ann2 :not :to-be nil)
+            (expect (treemacs-annotation->suffix ann1) :to-equal '(("Source2" . "Suffix2")))
+            (expect (treemacs-annotation->suffix ann2) :to-equal '(("Source3" . "Suffix3")))))))))
+(describe "treemacs--prefix-arg-to-recurse-depth"
+
+  (it "translates numbers literally"
+    (dotimes (n 10)
+      (expect (treemacs--prefix-arg-to-recurse-depth n) :to-be n)))
+
+  (it "translates nil to 0"
+      (expect (treemacs--prefix-arg-to-recurse-depth nil) :to-be 0))
+
+  (it "translates everything else to 999"
+    (expect (treemacs--prefix-arg-to-recurse-depth '(4)) :to-be 999)
+    (expect (treemacs--prefix-arg-to-recurse-depth 1.0) :to-be 999)
+    (expect (treemacs--prefix-arg-to-recurse-depth "a") :to-be 999)
+    (expect (treemacs--prefix-arg-to-recurse-depth (treemacs-project->create!)) :to-be 999)))
+
 (provide 'test-treemacs)
 
 ;;; treemacs-test.el ends here
